@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2023 Ralf Hoffmann.
+ * Copyright (C) 2001-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -191,13 +191,27 @@ public:
   void setCursor(Window win,int type);
   void unsetCursor(Window win);
 
-  enum {WAIT_CURSOR=0,SCROLLH_CURSOR,SCROLLV_CURSOR,MAXCURSORS};
+    enum {
+        WAIT_CURSOR = 0,
+        SCROLLH_CURSOR,
+        SCROLLV_CURSOR,
+        SCROLLFOUR_CURSOR,
+        DOUBLE_DOWN_CURSOR,
+        DOUBLE_UP_CURSOR,
+        DOUBLE_LEFT_CURSOR,
+        DOUBLE_RIGHT_CURSOR,
+        MAXCURSORS};
+
+    enum clipboard_request_e {
+        CLIPBOARD_REQUEST_STRING,
+        CLIPBOARD_REQUEST_FILE_LIST
+    };
+
   int startCut(GUIElement *elem,const char *buffer, bool use_clipboard);
   void cancelCut(bool use_clipboard);
-  int startPaste(GUIElement *elem, bool use_clipboard);
   bool amiOwner(bool use_clipboard) const;
   std::string getCutBuffer(bool use_clipboard) const;
-  void requestCut( Window win, bool fallback, bool use_clipboard );
+  void requestCut( Window win, bool fallback, bool use_clipboard, GUIElement *elem, void *agmessage_data, clipboard_request_e request_type );
   void rerequestCut();
   void cancelCutPaste(GUIElement *elem, bool use_clipboard);
   bool isDoubleClick(struct timeval *t1,struct timeval *t2) const;
@@ -255,6 +269,7 @@ public:
   bool msgHoldsLock( const Widget *e );
   void enableTimer();
   void disableTimer();
+  enum { TIMER_TICKS = 25 }; //HARDCODED
 
   int getTextWidth( const char *str, AGUIXFont *font = NULL );
   int getTextWidth( const char *str, AGUIXFont *font, int len );
@@ -321,6 +336,8 @@ public:
 
     void copyToClipboard( const std::string &s, enum target_clipboard_buffer target );
     void copyPrimaryToClipboard();
+    void copyFileListToClipboard( const std::vector< std::string > &l,
+                                  bool cut_operation );
 
     /**
      * return the location and dimension of the largest (xinerama) screen
@@ -425,7 +442,8 @@ private:
       XA_TEXT_PLAIN,
       XA_NET_WM_ICON,
       XA_CARDINAL_ATOM,
-      XA_CLIPBOARD;
+      XA_CLIPBOARD,
+      XA_SPECIAL_COPIED_FILES;
 
   int m_argc;
   char **m_argv;
@@ -501,7 +519,6 @@ private:
   struct timeval timerStart;
 #endif
   int msgHandler( int mode, AWindow *parent, bool onlyExpose );
-  enum { TIMER_TICKS = 25 }; //HARDCODED
   long lastTimerEventNr;
 
   void checkPopUpWindows( Message * );
@@ -530,6 +547,7 @@ private:
     void setTargetsProperty( Window w, Atom property );
     Window findXDNDAwareWindow( Window w, Window *return_proxy_window );
 
+    std::tuple< int, std::string, std::vector< std::string > > convertPasteDataToFileList( const unsigned char *data );
   int m_open_popup_counter;
   bool m_popup_ignore_button_release;
 
@@ -544,12 +562,27 @@ private:
 
     class copy_and_paste_state {
     public:
+        void reset() {
+            buffer_string.clear();
+            file_list.clear();
+            file_list_cut = false;
+            last_selection_request_atom = None;
+            last_selection_request_window = None;
+            cutstart = NULL;
+            paste_elem = NULL;
+            paste_agmessage_data = NULL;
+        }
+
         std::string buffer_string;
+        std::vector< std::string > file_list;
+        bool file_list_cut = false;
 
         Atom last_selection_request_atom = None;
         Window last_selection_request_window = None;
 
-        GUIElement *cutstart = NULL, *pastestart = NULL;
+        GUIElement *cutstart = NULL;
+        GUIElement *paste_elem = NULL;
+        void *paste_agmessage_data = NULL;
     };
 
     copy_and_paste_state m_primary_buffer_state;
