@@ -317,7 +317,18 @@ bool AWindow::handleMessage(XEvent *E,Message *msg)
        ( msg->type != FocusIn ) &&
        ( msg->type != FocusOut ) ) {
     // but only without lockElement or we don't own it
-    if ( conLock == false ) return returnvalue;
+      if ( conLock == false ) {
+          // pretend we handled this event
+          if ( isParent( msg->window, false ) == true ) {
+              if ( msg->type == KeyRelease ||
+                   msg->type == KeyPress ||
+                   msg->type == ButtonPress ||
+                   msg->type == ButtonRelease ) {
+                  return true;
+              }
+          }
+          return returnvalue;
+      }
   }
 
   if ( msg->loop == 0 ) {
