@@ -1,6 +1,6 @@
 /* awindow.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2022 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -147,7 +147,7 @@ public:
   void removeContainer( AContainer *cont );
   void removeFromContainer( Widget *twid );
 
-  void contMaximize( bool applyMinSize = false, bool applyMaxSize = false );
+  void contMaximize( bool applyMinSize = false, bool applyMaxSize = false, bool grow_only = false );
   void updateCont();
 
   template <class T> T *add( T *w )
