@@ -1,6 +1,6 @@
 /* fieldlistview.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2022 Ralf Hoffmann.
+ * Copyright (C) 2002-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -32,6 +32,7 @@
 #include <X11/X.h>
 #include <set>
 #include <functional>
+#include "pan_scrolling.hh"
 
 class AGUIX;
 class AGUIXFont;
@@ -174,6 +175,8 @@ public:
 
   typedef enum { MOUSE_SELECT_NORMAL, MOUSE_SELECT_ALT } mouse_select_t;
   static int changeSelectMode( mouse_select_t nm );
+    static int changeHScrollMode( bool nv );
+    static void changeDelayedEntryPressedEvent( bool nv );
   
   class ColorDef
   {
@@ -249,6 +252,8 @@ public:
     int getRowYPosition( int row );
 
     void setShowSelectionInVBar( bool nv );
+
+    void setHideEmptyField( int field, bool v );
 protected:
   int fields;
   int *fieldwidth;
@@ -262,6 +267,7 @@ protected:
 
   std::vector< int > m_header_text_width;
   std::vector< int > m_field_spacing;
+    std::vector< bool > m_hide_empty_field;
 
   int fieldtextclicked;
 
@@ -451,6 +457,9 @@ protected:
   static unsigned int _scroll_button, _scroll_buttonmask, _scroll_mod;
   static unsigned int _entry_pressed_msg_button, _entry_pressed_msg_buttonmask, _entry_pressed_msg_mod;
   static mouse_select_t _select_mode;
+    static unsigned int s_hscroll_left_button;
+    static unsigned int s_hscroll_right_button;
+    static bool s_enable_delayed_entry_pressed_event;
 
   unsigned int getCurrentMouseButtonState( Window twin );
   
@@ -499,6 +508,11 @@ protected:
     int m_header_border_leftright = 3;
     int m_header_border_topbottom = 2;
 
+    pan_info m_pan_info;
+
+    Time m_middle_first_timer_event = 0;
+    bool m_middle_delayed_entry_pressed_disabled = false;
+
     void drawSelectStateVBar();
     void freeVBarSelectBuffer();
     int checkVBarSelectBuffer( int tw, int th );
