@@ -1,6 +1,6 @@
 /* request.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -240,6 +240,28 @@ int Requester::string_request( const char *title, const char *text, const char *
 }
 
 int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags)
+{
+    request_options options;
+
+    return string_request( title, text, default_str,
+                           buttons, return_str, twin, flags, options );
+}
+
+int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags,
+                               struct request_options &options )
+{
+    return string_request( title, text, default_str,
+                           buttons, return_str, twin, flags,
+                           options,
+                           "", nullptr );
+}
+
+int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons,
+                               char **return_str, const AWindow *twin, request_flags_t flags,
+                               struct request_options &options,
+                               const std::string &help_text,
+                               const std::function< std::string ( const std::string &current_input_before_cursor,
+                                                                  const std::string &current_input_after_cursor ) > help_callback )
 {
   Button **buts;
   int nbuttons;
@@ -261,7 +283,7 @@ int Requester::string_request( const char *title, const char *text, const char *
     win->setTransientForAWindow( transientawin );
   }
 
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 4 ), true );
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 4 + ( help_callback ? 1 : 0 ) ), true );
   ac1->setMinSpace( 0 );
   ac1->setMaxSpace( 0 );
   ac1->setBorderWidth( 5 );
@@ -274,16 +296,30 @@ int Requester::string_request( const char *title, const char *text, const char *
                        0, 1, AContainer::CO_INCW );
   sg->setAcceptFocus( true );
 
-  ac1->setMinHeight( 5, 0, 2 );
+  AContainerBB *ac1_help = nullptr;
+  Text *help_text_w = nullptr;
+
+  if ( help_callback ) {
+      ac1_help = dynamic_cast<AContainerBB*>( ac1->add( new AContainerBB( win, 2, 1 ), 0, 2 ) );
+      ac1_help->setBorderWidth( 5 );
+      ac1_help->setMinSpace( 5 );
+      ac1_help->setMaxSpace( 5 );
+      ac1_help->addWidget( new Text( aguix, 0, 0, help_text.c_str(), 1, 0 ),
+                           0, 0, AContainer::CO_FIX );
+      help_text_w = ac1_help->addWidget( new Text( aguix, 0, 0, help_callback( "", "" ).c_str(), 1, 0 ),
+                                       1, 0, AContainer::CO_INCW );
+  }
+
+  ac1->setMinHeight( 5, 0, 2 + ( help_callback ? 1 : 0 ) );
 
   AContainer *ac1_buttons = NULL;
 
   if ( nbuttons == 1 ) {
-    ac1_buttons = ac1->add( new AContainer( win, 3, 1 ), 0, 3 );
+    ac1_buttons = ac1->add( new AContainer( win, 3, 1 ), 0, 3 + ( help_callback ? 1 : 0 ) );
     ac1_buttons->setMinSpace( 0 );
     ac1_buttons->setMaxSpace( 0 );
   } else {
-    ac1_buttons = ac1->add( new AContainer( win, nbuttons, 1 ), 0, 3 );
+    ac1_buttons = ac1->add( new AContainer( win, nbuttons, 1 ), 0, 3 + ( help_callback ? 1 : 0 ) );
     ac1_buttons->setMinSpace( 5 );
     ac1_buttons->setMaxSpace( -1 );
   }
@@ -316,6 +352,12 @@ int Requester::string_request( const char *title, const char *text, const char *
     }
   }
 
+  if ( options.select_characters < 0 ) {
+      sg->selectAll();
+  } else if ( options.select_characters > 0 ) {
+      sg->setSelection( 0, options.select_characters - 1, false );
+  }
+
   sg->takeFocus();
   win->useStippleBackground();
   win->setDoTabCycling( true );
@@ -357,6 +399,19 @@ int Requester::string_request( const char *title, const char *text, const char *
 	    }
 	  }
 	  break;
+    case AG_STRINGGADGET_CONTENTCHANGE:
+    case AG_STRINGGADGET_CURSORCHANGE:
+        if ( help_callback ) {
+            auto current_input = sg->getText();
+            int cursor = sg->getCursor();
+            std::string before( current_input, cursor );
+            auto res = help_callback( before, "" );
+            help_text_w->setText( res.c_str() );
+
+            ac1_help->readLimits();
+            win->contMaximize( true, false, true );
+        }
+        break;
 	case AG_KEYPRESSED:
 	  if ( win->isParent( msg->key.window, false ) == true ) {
 	    // key release only accept when not first key we get
@@ -641,17 +696,17 @@ int Requester::request_choose( const char *title,
 }
 
 
-int Requester::request( const char *title, TextStorage &ts, const char *buttons )
+int Requester::request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons )
 {
   return request( title, ts, buttons, REQUEST_NONE );
 }
 
-int Requester::request( const char *title, TextStorage &ts, const char *buttons, request_flags_t flags )
+int Requester::request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons, request_flags_t flags )
 {
   return request( title, ts, buttons, NULL, flags );
 }
 
-int Requester::request( const char *title, TextStorage &ts, const char *buttons, const AWindow *twin, request_flags_t flags )
+int Requester::request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons, const AWindow *twin, request_flags_t flags )
 {
   int i;
   Button **buts;
