@@ -1,6 +1,6 @@
 /* request.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,6 +24,8 @@
 
 #include "aguixdefs.h"
 #include "textstorage.h"
+#include <functional>
+#include <string>
 
 class Requester
 {
@@ -40,12 +42,24 @@ public:
 		 REQUEST_SELECTALL = 3,
 		 REQUEST_CANCELWITHLEFT = 4 } request_flags_t;
 
+    struct request_options {
+        ssize_t select_characters = -1; ///< -1 means all, >= 0 selects number of character
+    };
+
   int request(const char *title,const char *text,const char *buttons);
   int request(const char *title,const char *text,const char *buttons, request_flags_t flags);
   int request(const char *title,const char *text,const char *buttons, const AWindow *twin, request_flags_t flags);
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str);
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, request_flags_t flags );
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags );
+  int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags,
+                      struct request_options &options );
+  int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,
+                      char **return_str, const AWindow *twin, request_flags_t flags,
+                      struct request_options &options,
+                      const std::string &help_text,
+                      const std::function< std::string ( const std::string &current_input_before_cursor,
+                                                         const std::string &current_input_after_cursor ) > help_callback );
   int request_choose( const char *title, const char *text, const char *choose_text, bool &choose_var, const char *buttons );
   int request_choose( const char *title, const char *text, const char *choose_text, bool &choose_var, const char *buttons, request_flags_t flags );
   int request_choose( const char *title, const char *text, const char *choose_text, bool &choose_var, const char *buttons, const AWindow *twin, request_flags_t flags );
@@ -75,9 +89,9 @@ public:
                              const AWindow *twin,
                              request_flags_t flags );
 
-  int request( const char *title, TextStorage &ts, const char *buttons );
-  int request( const char *title, TextStorage &ts, const char *buttons, request_flags_t flags );
-  int request( const char *title, TextStorage &ts, const char *buttons, const AWindow *twin, request_flags_t flags );
+  int request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons );
+  int request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons, request_flags_t flags );
+  int request( const char *title, std::shared_ptr< TextStorage > ts, const char *buttons, const AWindow *twin, request_flags_t flags );
 private:
   class AGUIX *aguix;
   AWindow *transientawin;
