@@ -856,8 +856,11 @@ void StringGadget::insertSelection( bool from_clipboard )
             setCursor( content.getCursorPos() + s2.length() );
         }
     } else {
-        _aguix->requestCut( getWindow(), false, from_clipboard );
-        _aguix->startPaste( this, from_clipboard );
+        _aguix->requestCut( getWindow(),
+                            false,
+                            from_clipboard,
+                            this, NULL,
+                            AGUIX::CLIPBOARD_REQUEST_STRING );
         wantpaste = true;
         pasterequest = time( NULL );
         paste_from_clipboard = from_clipboard;
@@ -962,7 +965,7 @@ void StringGadget::setStrongKeyCapture(bool nv)
   strongkeycapture=nv;
 }
 
-void StringGadget::paste(unsigned char*buf)
+void StringGadget::paste( const unsigned char *buf )
 {
   if((wantpaste==true)&&(difftime(time(NULL),pasterequest)<10.0)) {
     if(forbidPosChange==false) {
