@@ -24,6 +24,12 @@
 #include "drawablecont.hh"
 #include "utf8.hh"
 
+unsigned int TextView::s_scroll_button = Button3;
+unsigned int TextView::s_scroll_buttonmask = Button3Mask;
+unsigned int TextView::s_scroll_mod = 0;
+
+const static int s_pan_prepare_delta = 10;
+
 TVCallBack::TVCallBack( TextView *_tv )
 {
   tv = _tv;
@@ -39,18 +45,19 @@ void TVCallBack::run_int( Widget *elem, int value )
 }
 
 TextView::TextView( AGUIX *parent,
-		    int x,
-		    int y,
-		    int width,
-		    int height,
-		    std::string title,
-		    TextStorage &_ts ) : AWindow( parent, x, y, width, height, title ),
-                                 ts( _ts ),
-                                 tvcb( this ),
-                                 _show_line_numbers( false ),
-                                 m_buffer( 0 ),
-                                 m_show_frame( true ),
-                                 m_last_mouse_time( 0 )
+                    int x,
+                    int y,
+                    int width,
+                    int height,
+                    std::string title,
+                    std::shared_ptr< TextStorage > ts ) :
+    AWindow( parent, x, y, width, height, title ),
+    m_ts( ts ),
+    tvcb( this ),
+    _show_line_numbers( false ),
+    m_buffer( 0 ),
+    m_show_frame( true ),
+    m_last_mouse_time( 0 )
 {
   hbar = vbar = NULL;
   cont = NULL;
@@ -71,6 +78,11 @@ TextView::TextView( AGUIX *parent,
   gettimeofday( &_lastwheel, NULL );
 
   initColors();
+
+  if ( ! m_ts ) {
+      m_ts = std::make_shared< TextStorageString >( "",
+                                                    std::make_shared< ACharWidth >() );
+  }
 }
 
 TextView::~TextView()
@@ -183,17 +195,27 @@ bool TextView::handleMessage(XEvent *E,Message *msg)
   if ( isCreated() == false ) return false;
   
   if ( msg->lockElement == this &&
-       m_selection_scroll_active ) {
+       ( m_selection_scroll_active ||
+         m_pan_info.m_pan_mode != m_pan_info.PAN_INACTIVE ) ) {
       msg->ack = true;
   }
 
   if ( msg->type == Expose &&
        msg->window == win ) {
       drawBuffer( msg->x, msg->y, msg->width, msg->height );
+      return AWindow::handleMessage( E, msg );
   } else if ( msg->type == MapNotify &&
-       msg->window == win ) {
+              msg->window == win ) {
       redraw();
-  } else if ( msg->type == KeyPress ) {
+      return AWindow::handleMessage( E, msg );
+  }
+
+  if ( m_pan_info.m_pan_mode != m_pan_info.PAN_INACTIVE ) {
+      handleThird( msg );
+      return AWindow::handleMessage( E, msg );
+  }
+  
+  if ( msg->type == KeyPress ) {
     if ( ( getAcceptFocus() == true ) && ( getHasFocus() == true ) ) {
       if ( isVisible() == true ) {
 	if ( isTopParent( msg->window ) == true ) {
@@ -251,6 +273,8 @@ bool TextView::handleMessage(XEvent *E,Message *msg)
       _aguix->Flush();
       _aguix->msgLock( this );
       _aguix->enableTimer();
+    } else if ( msg->button == s_scroll_button ) {
+        handleThird( msg );
     }
   } else if ( msg->type == ButtonRelease && msg->window == win ) {
       if ( msg->button == Button1 ) {
@@ -311,6 +335,84 @@ bool TextView::handleMessage(XEvent *E,Message *msg)
   return AWindow::handleMessage( E, msg );
 }
 
+void TextView::handleThird(Message *msg)
+{
+    int mx, my;
+
+    if ( isCreated() == false ) return;
+
+    if ( msg->type == ButtonPress ) {
+        if ( msg->window != win ) return;
+        if ( m_pan_info.m_pan_mode != m_pan_info.PAN_INACTIVE ) return;
+
+        mx = msg->mousex;
+        my = msg->mousey;
+
+        m_pan_info.m_pan_mode = m_pan_info.PAN_PREPARE;
+        m_pan_info.m_pan_base_mx = mx;
+        m_pan_info.m_pan_base_my = my;
+        _aguix->setCursor( win, AGUIX::SCROLLFOUR_CURSOR );
+        _aguix->Flush();
+        _aguix->msgLock( this );
+    } else if ( msg->type == MotionNotify ) {
+        if ( m_pan_info.m_pan_mode != m_pan_info.PAN_INACTIVE ) {
+            _aguix->queryPointer( win, &mx, &my );
+
+            if ( m_pan_info.m_pan_mode == m_pan_info.PAN_PREPARE ) {
+                if ( abs( m_pan_info.m_pan_base_mx - mx ) > s_pan_prepare_delta ||
+                     abs( m_pan_info.m_pan_base_my - my ) > s_pan_prepare_delta ) {
+                    int iw = cont->getWidth( 0, 0 ) - 2;
+                    if ( iw < 0 ) iw = 0;
+                    int ih = cont->getHeight( 0, 0 ) - 2;
+                    if ( ih < 0 ) ih = 0;
+
+                    if ( abs( m_pan_info.m_pan_base_mx - mx ) > abs( m_pan_info.m_pan_base_my - my ) ) {
+                        m_pan_info.m_pan_mode = m_pan_info.PAN_HORIZONTAL;
+
+                        _aguix->setCursor( win, AGUIX::SCROLLH_CURSOR );
+
+                        // mouse position areas
+                        m_pan_info.init_mouse_positions( mx, iw );
+
+                        // scroll positions
+                        m_pan_info.init_scroll_positions( hbar->getOffset(),
+                                                          hbar->getMaxLen() - hbar->getMaxDisplay(),
+                                                          hbar->getMaxDisplay() );
+                    } else {
+                        m_pan_info.m_pan_mode = m_pan_info.PAN_VERTICAL;
+
+                        _aguix->setCursor( win, AGUIX::SCROLLV_CURSOR );
+
+                        // mouse position areas
+                        m_pan_info.init_mouse_positions( my, ih );
+
+                        // scroll positions
+                        m_pan_info.init_scroll_positions( vbar->getOffset(),
+                                                          vbar->getMaxLen() - vbar->getMaxDisplay(),
+                                                          vbar->getMaxDisplay() );
+                    }
+                }
+            }
+
+            if ( m_pan_info.m_pan_mode == m_pan_info.PAN_VERTICAL ) {
+                vbar->setOffset( m_pan_info.get_scroll_offset( m_pan_info.m_pan_mode, mx, my ) );
+            } else if ( m_pan_info.m_pan_mode == m_pan_info.PAN_HORIZONTAL ) {
+                hbar->setOffset( m_pan_info.get_scroll_offset( m_pan_info.m_pan_mode, mx, my ) );
+            }
+            redraw();
+            checkForEndReached();
+        }
+    } else if ( msg->type == ButtonRelease ) {
+        if ( msg->window == win &&
+             msg->button == s_scroll_button &&
+             m_pan_info.m_pan_mode != m_pan_info.PAN_INACTIVE ) {
+            m_pan_info.m_pan_mode = m_pan_info.PAN_INACTIVE;
+            _aguix->unsetCursor( win );
+            _aguix->msgUnlock( this );
+        }
+    }
+}
+
 void TextView::doDestroyStuff()
 {
   if ( m_buffer != 0 ) {
@@ -401,12 +503,12 @@ void TextView::forceRedraw()
   getLinesWidth( lines, width );
 
   if ( ( _w != last_w ) || ( _h != last_h ) || ( width != last_cont_width ) ) {
-    real_line = ts.getRealLinePair( vbar->getOffset() );
+    real_line = m_ts->getRealLinePair( vbar->getOffset() );
 
     if ( line_wrap == true ) {
-      ts.setLineLimit( width );
+      m_ts->setLineLimit( width );
     } else {
-      ts.setLineLimit( -1 );
+      m_ts->setLineLimit( -1 );
     }
     updateBars();
     last_w = _w;
@@ -414,7 +516,7 @@ void TextView::forceRedraw()
     last_cont_width = width;
 
     if ( _highlight_line.real_line_number >= 0 ) {
-        _highlight_line.y_offset = ts.findLineNr( std::pair<int,int>( _highlight_line.real_line_number, 0 ) );
+        _highlight_line.y_offset = m_ts->findLineNr( std::pair<int,int>( _highlight_line.real_line_number, 0 ) );
     }
   } else {
       // just update the horizontal slider
@@ -422,13 +524,13 @@ void TextView::forceRedraw()
   }
 
   if ( real_line.first >= 0 ) {
-    int linenr = ts.findLineNr( real_line );
+    int linenr = m_ts->findLineNr( real_line );
 
     if ( linenr == 0 && ( real_line.first != 0 || real_line.second != 0 ) ) {
       // wasn't able to found line, probably the subline doesn't exists anymore because of
       // rewrapping so try subline 0
       real_line.second = 0;
-      linenr = ts.findLineNr( real_line );
+      linenr = m_ts->findLineNr( real_line );
     }
     vbar->setOffset( linenr );
   }
@@ -449,9 +551,9 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
     if ( font == NULL ) elementHeight = _aguix->getCharHeight();
     else elementHeight = font->getCharHeight();
 
-    std::pair< std::pair< int, int >, int > actual_sel_start = ts.getLineForOffset( m_selection.start_uline,
+    std::pair< std::pair< int, int >, int > actual_sel_start = m_ts->getLineForOffset( m_selection.start_uline,
                                                                                     m_selection.start_offset );
-    std::pair< std::pair< int, int >, int > actual_sel_end = ts.getLineForOffset( m_selection.end_uline,
+    std::pair< std::pair< int, int >, int > actual_sel_end = m_ts->getLineForOffset( m_selection.end_uline,
                                                                                   m_selection.end_offset );
 
     if ( linePairGT( actual_sel_start.first, actual_sel_end.first ) == true ||
@@ -471,7 +573,7 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
         // starting at offset 0, width is not enough
         // possible values are -1 or width+hbar->getOffset()
         // (was width)
-        ts.getLine( vbar->getOffset() + i, 0, -1, s1 );
+        m_ts->getLine( vbar->getOffset() + i, 0, -1, s1 );
         
         int fg = _colors.getTextColor();
         if ( _highlight_line.real_line_number >= 0 &&
@@ -490,12 +592,12 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
         
         // ask how much bytes fit into the invisible area (due to horizontal scrolling)
         int hidden_length, hidden_width;
-        hidden_length = ts.getAWidth().getStrlen4WidthMaxlen( s1.c_str(), hbar->getOffset() / 2, hbar->getOffset(), &hidden_width );
+        hidden_length = m_ts->getAWidth().getStrlen4WidthMaxlen( s1.c_str(), hbar->getOffset() / 2, hbar->getOffset(), &hidden_width );
         
         int overscan_width = width + ( hbar->getOffset() - hidden_width );
-        int visible_length = ts.getAWidth().getStrlen4WidthMaxlen( s1.c_str() + hidden_length, overscan_width / 2, overscan_width, NULL );
+        int visible_length = m_ts->getAWidth().getStrlen4WidthMaxlen( s1.c_str() + hidden_length, overscan_width / 2, overscan_width, NULL );
         
-        std::pair<int,int> cur_line = ts.getRealLinePair( vbar->getOffset() + i );
+        std::pair<int,int> cur_line = m_ts->getRealLinePair( vbar->getOffset() + i );
 
         // line can be outside ( cur_line < actual_sel_start or cur_line > actual_sel_end )
         // line can be inside ( cur_line > actual_sel_start and cur_line < actual_sel_end )
@@ -530,9 +632,9 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                                           start_x - hbar->getOffset() + hidden_width, i * elementHeight + start_y, fg );
                         
                         hidden_length += s2.length();
-                        hidden_width += ts.getAWidth().getWidth( s2.c_str() );
+                        hidden_width += m_ts->getAWidth().getWidth( s2.c_str() );
                         visible_length -= s2.length();
-                        overscan_width -= ts.getAWidth().getWidth( s2.c_str() );
+                        overscan_width -= m_ts->getAWidth().getWidth( s2.c_str() );
                     } else {
                         // invalid selection start offset
                         // just draw the whole visible string
@@ -542,9 +644,9 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                                           start_x - hbar->getOffset() + hidden_width, i * elementHeight + start_y, fg );
                         
                         hidden_length += s2.length();
-                        hidden_width += ts.getAWidth().getWidth( s2.c_str() );
+                        hidden_width += m_ts->getAWidth().getWidth( s2.c_str() );
                         visible_length -= s2.length();
-                        overscan_width -= ts.getAWidth().getWidth( s2.c_str() );
+                        overscan_width -= m_ts->getAWidth().getWidth( s2.c_str() );
                     }
                 }
                 
@@ -565,7 +667,7 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                     
                     if ( selection_len > 0 ) {
                         std::string s2( s1, hidden_length, selection_len );
-                        int selection_width = ts.getAWidth().getWidth( s2.c_str() );
+                        int selection_width = m_ts->getAWidth().getWidth( s2.c_str() );
                         int max_line_width;
                         int rectangle_x = start_x - hbar->getOffset() + hidden_width;
 
@@ -635,11 +737,11 @@ void TextView::drawLineNumbers( int start_x, int end_x, int start_y, int lines )
         _aguix->setClip( font, &dc, start_x, 0, end_x, getHeight() );
         
         for ( int i = 0; i < lines; i++ ) {
-            real_line = ts.getRealLinePair( vbar->getOffset() + i );
+            real_line = m_ts->getRealLinePair( vbar->getOffset() + i );
             if ( real_line.second == 0 ) {
                 sprintf( buf, "%d", real_line.first + 1 );
                 
-                w = ts.getAWidth().getWidth( buf );
+                w = m_ts->getAWidth().getWidth( buf );
                 
                 _aguix->DrawText( dc, font, buf, end_x - w - 5, i * elementHeight + start_y, fg );
             }
@@ -666,7 +768,7 @@ void TextView::updateBars()
   
   vbar->setMaxDisplay( lines );
   hbar->setMaxDisplay( width );
-  vbar->setMaxLen( ts.getNrOfLines() );
+  vbar->setMaxLen( m_ts->getNrOfLines() );
   updateHBarForVisibleLines();
 }
 
@@ -728,7 +830,7 @@ void TextView::maximizeX( int max_width )
   
   bw = ( _display_focus == true ) ? 2 : 1;
 
-  l = ts.getMaxLineWidth() + 5;
+  l = m_ts->getMaxLineWidth() + 5;
   l += 2 * bw + 2 + ( ( vbar != NULL ) ? vbar->getWidth() : 0 );
   l += getTextStartOffset();
 
@@ -757,13 +859,13 @@ void TextView::maximizeYLines( int max_lines, int width )
 
   if ( width > 0 ) {
       if ( line_wrap == true ) {
-          ts.setLineLimit( width );
+          m_ts->setLineLimit( width );
       } else {
-          ts.setLineLimit( -1 );
+          m_ts->setLineLimit( -1 );
       }
   }
 
-  l = ts.getNrOfLines();
+  l = m_ts->getNrOfLines();
   if ( l > max_lines ) l = max_lines;
   l = ( l + 1 ) * elementHeight;
   l += 2 * bw + 2 + ( ( ( hbar != NULL ) && ( line_wrap == false ) ) ? hbar->getHeight() : 0 );
@@ -796,14 +898,14 @@ int TextView::getTextStartOffset() const
   if ( _show_line_numbers == false )
     return off;
 
-  int lines = ts.getNrOfLines();
+  int lines = m_ts->getNrOfLines();
   int digits = 1;
   if ( lines > 0 ) {
     digits = (int)( log( (double)lines ) / log( 10.0 ) ) ;
     digits++;
   }
   std::string maxnumber( digits, '9' );
-  off = ts.getAWidth().getWidth( maxnumber.c_str() );
+  off = m_ts->getAWidth().getWidth( maxnumber.c_str() );
   off += 5;
   return off;
 }
@@ -824,7 +926,7 @@ bool TextView::getShowLineNumbers() const
 void TextView::jumpToLine( int line_nr, bool highlight_line )
 {
   int l;
-  l = ts.findLineNr( std::pair<int,int>( line_nr, 0 ) );
+  l = m_ts->findLineNr( std::pair<int,int>( line_nr, 0 ) );
 
   if ( highlight_line == true ) {
     _highlight_line.real_line_number = line_nr;
@@ -945,13 +1047,23 @@ void TextView::textStorageChanged()
     redraw();
 }
 
+void TextView::replaceTextStorage( std::shared_ptr< TextStorage > ts )
+{
+    m_ts = ts;
+    if ( ! m_ts ) {
+        m_ts = std::make_shared< TextStorageString >( "",
+                                                      std::make_shared< ACharWidth >() );
+    }
+    textStorageChanged();
+}
+
 void TextView::updateHBarForVisibleLines()
 {
     int lines, width, maxlen = 0;
     getLinesWidth( lines, width );
 
     for ( int i = 0; i < lines; i++ ) {
-        int l = ts.getLineWidth( i + vbar->getOffset() );
+        int l = m_ts->getLineWidth( i + vbar->getOffset() );
         if ( l > maxlen ) {
             maxlen = l;
         }
@@ -963,7 +1075,7 @@ void TextView::updateHBarForVisibleLines()
 
 void TextView::setSelectionStart( int unwrapped_line_nr, int offset )
 {
-    if ( unwrapped_line_nr >= 0 && unwrapped_line_nr < ts.getNrOfUnwrappedLines() ) {
+    if ( unwrapped_line_nr >= 0 && unwrapped_line_nr < m_ts->getNrOfUnwrappedLines() ) {
         m_selection.start_uline = unwrapped_line_nr;
         m_selection.start_offset = offset;
     } else {
@@ -979,7 +1091,7 @@ void TextView::getSelectionStart( int &unwrapped_line_nr, int &offset ) const
 
 void TextView::setSelectionEnd( int unwrapped_line_nr, int offset )
 {
-    if ( unwrapped_line_nr >= 0 && unwrapped_line_nr < ts.getNrOfUnwrappedLines() ) {
+    if ( unwrapped_line_nr >= 0 && unwrapped_line_nr < m_ts->getNrOfUnwrappedLines() ) {
         m_selection.end_uline = unwrapped_line_nr;
         m_selection.end_offset = offset;
     } else {
@@ -1005,9 +1117,9 @@ void TextView::selectAll()
 
     std::string line_str;
 
-    ts.getLine( ts.getNrOfLines() - 1, 0, line_str );
+    m_ts->getLine( m_ts->getNrOfLines() - 1, 0, line_str );
 
-    setSelectionEnd( ts.getNrOfUnwrappedLines() - 1, (int)line_str.size() );
+    setSelectionEnd( m_ts->getNrOfUnwrappedLines() - 1, (int)line_str.size() );
 }
 
 void TextView::sizeChanged( int width, int height, bool explicit_resize )
@@ -1020,9 +1132,9 @@ void TextView::sizeChanged( int width, int height, bool explicit_resize )
 
 void TextView::makeSelectionVisible()
 {
-    std::pair< std::pair< int, int >, int > actual_sel_start = ts.getLineForOffset( m_selection.start_uline,
+    std::pair< std::pair< int, int >, int > actual_sel_start = m_ts->getLineForOffset( m_selection.start_uline,
                                                                                     m_selection.start_offset );
-    std::pair< std::pair< int, int >, int > actual_sel_end = ts.getLineForOffset( m_selection.end_uline,
+    std::pair< std::pair< int, int >, int > actual_sel_end = m_ts->getLineForOffset( m_selection.end_uline,
                                                                                   m_selection.end_offset );
 
     if ( linePairGT( actual_sel_start.first, actual_sel_end.first ) == true ||
@@ -1038,7 +1150,7 @@ void TextView::makeSelectionVisible()
     }
 
     int l;
-    l = ts.findLineNr( std::pair<int,int>( actual_sel_start.first.first, actual_sel_start.first.second ) );
+    l = m_ts->findLineNr( std::pair<int,int>( actual_sel_start.first.first, actual_sel_start.first.second ) );
     
     if ( l >=0 && l < vbar->getMaxLen() ){
         if ( l < vbar->getOffset() ) {
@@ -1053,14 +1165,14 @@ void TextView::makeSelectionVisible()
             // need to update hbar to be able to set valid offsets
             updateHBarForVisibleLines();
 
-            if ( ts.getLine( l, 0, actual_sel_start.second, str1 ) == 0 ) {
-                int selstart_offset = ts.getAWidth().getWidth( str1.c_str() );
+            if ( m_ts->getLine( l, 0, actual_sel_start.second, str1 ) == 0 ) {
+                int selstart_offset = m_ts->getAWidth().getWidth( str1.c_str() );
                 int selend_offset = -1;
                 int new_offset = hbar->getOffset();
                 
                 if ( linePairEQ( actual_sel_start.first, actual_sel_end.first ) == true ) {
-                    if ( ts.getLine( l, 0, actual_sel_end.second, str1 ) == 0 ) {
-                        selend_offset = ts.getAWidth().getWidth( str1.c_str() );
+                    if ( m_ts->getLine( l, 0, actual_sel_end.second, str1 ) == 0 ) {
+                        selend_offset = m_ts->getAWidth().getWidth( str1.c_str() );
                     }
                 }
 
@@ -1268,15 +1380,15 @@ std::pair< int, int > TextView::getTextPosForMousePos( int x, int y )
 
     bool bottom_outside = false;
 
-    if ( absolute_row >= ts.getNrOfLines() ) {
-        absolute_row = ts.getNrOfLines() - 1;
+    if ( absolute_row >= m_ts->getNrOfLines() ) {
+        absolute_row = m_ts->getNrOfLines() - 1;
         bottom_outside = true;
     } else if ( relative_row >= vbar->getMaxDisplay() ) {
         absolute_row = vbar->getOffset() + vbar->getMaxDisplay() - 1;
         bottom_outside = true;
     }
     
-    std::pair<int,int> cur_line = ts.getRealLinePair( absolute_row );
+    std::pair<int,int> cur_line = m_ts->getRealLinePair( absolute_row );
 
     if ( cur_line.first < 0 ) return cur_line;
 
@@ -1286,18 +1398,18 @@ std::pair< int, int > TextView::getTextPosForMousePos( int x, int y )
         for ( int sl = 0; sl < cur_line.second; sl++ ) {
             std::string s;
 
-            ts.getLine( absolute_row - cur_line.second + sl, 0, s );
+            m_ts->getLine( absolute_row - cur_line.second + sl, 0, s );
             prev_sublines_chars += s.size();
         }
     }
 
     std::string clicked_line;
 
-    ts.getLine( absolute_row, 0, clicked_line );
+    m_ts->getLine( absolute_row, 0, clicked_line );
 
     int offset_length;
     if ( ! bottom_outside ) {
-        offset_length = ts.getAWidth().getStrlen4WidthMaxlen( clicked_line.c_str(), base_x / 2, base_x, NULL );
+        offset_length = m_ts->getAWidth().getStrlen4WidthMaxlen( clicked_line.c_str(), base_x / 2, base_x, NULL );
     } else {
         offset_length = (int)clicked_line.size();
     }
@@ -1318,18 +1430,18 @@ void TextView::copySelectionToClipboard()
     if ( unwrapped_start_line >= 0 && unwrapped_end_line >= unwrapped_start_line ) {
 
         if ( unwrapped_start_line == unwrapped_end_line ) {
-            ts.getUnwrappedLineRaw( unwrapped_start_line,
+            m_ts->getUnwrappedLineRaw( unwrapped_start_line,
                                     unwrapped_start_offset,
                                     unwrapped_end_offset - unwrapped_start_offset, s );
         } else {
-            ts.getUnwrappedLineRaw( unwrapped_start_line,
+            m_ts->getUnwrappedLineRaw( unwrapped_start_line,
                                     unwrapped_start_offset, s );
 
             std::string temp_line_str;
 
             // all full lines between start and end
             for ( int l = unwrapped_start_line + 1; l < unwrapped_end_line; l++ ) {
-                ts.getUnwrappedLineRaw( l, 0, temp_line_str );
+                m_ts->getUnwrappedLineRaw( l, 0, temp_line_str );
 
                 s += '\n';
 
@@ -1337,7 +1449,7 @@ void TextView::copySelectionToClipboard()
             }
 
             // and the rest up to the end offset
-            ts.getUnwrappedLineRaw( unwrapped_end_line, 0, unwrapped_end_offset, temp_line_str );
+            m_ts->getUnwrappedLineRaw( unwrapped_end_line, 0, unwrapped_end_offset, temp_line_str );
 
             s += '\n';
 
@@ -1358,7 +1470,7 @@ void TextView::handleDoubleClick( int mx, int my )
     std::pair< int, int > cur = getTextPosForMousePos( mx, my );
     std::string line_str;
 
-    if ( ts.getUnwrappedLine( cur.first, 0, -1, line_str ) != 0 ) {
+    if ( m_ts->getUnwrappedLine( cur.first, 0, -1, line_str ) != 0 ) {
         return;
     }
 
@@ -1494,3 +1606,16 @@ void TextView::getTextArea( int &x, int &y,
         h = 0;
     }
 }
+
+bool TextView::isAtEnd() const
+{
+    return vbar->getOffset() + vbar->getMaxDisplay() >= vbar->getMaxLen();
+}
+
+void TextView::scrollToEnd()
+{
+    if ( vbar != NULL ) {
+		setVBarOffset( vbar->getMaxLen() );
+		redraw();
+    }
+}
