@@ -1,6 +1,6 @@
 /* utf8.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2017 Ralf Hoffmann.
+ * Copyright (C) 2006-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -289,6 +289,11 @@ std::string UTF8::convertToValidCharacterString( const char *str )
   
     while ( pos < len ) {
         if ( isValidCharacter( str + pos ) == false ) {
+            res += "\\x";
+            unsigned char nibble0 = (unsigned char)str[pos] >> 4;
+            unsigned char nibble1 = (unsigned char)str[pos] & 0xf;
+            res += nibble0 <= 9 ? ( '0' + nibble0 ) : ( 'a' + nibble0 - 10 );
+            res += nibble1 <= 9 ? ( '0' + nibble1 ) : ( 'a' + nibble1 - 10 );
             pos++;
             continue;
         }
