@@ -1,6 +1,6 @@
 /* basic_actions.h
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -33,13 +33,10 @@ public:
   DNDAction( const DNDAction &other );
   DNDAction &operator=( const DNDAction &other );
 
-  virtual DNDAction *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
+  DNDAction *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
   virtual const char *getDescription();
 
   class DNDActionDescr : public ActionDescr
@@ -60,14 +57,11 @@ public:
   DoubleClickAction( const DoubleClickAction &other );
   DoubleClickAction &operator=( const DoubleClickAction &other );
 
-  virtual DoubleClickAction *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual const char *getDescription();
+  DoubleClickAction *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  const char *getDescription() override;
 
   class DoubleClickActionDescr : public ActionDescr
   {
@@ -87,14 +81,11 @@ public:
   ShowAction( const ShowAction &other );
   ShowAction &operator=( const ShowAction &other );
 
-  virtual ShowAction *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual const char *getDescription();
+  ShowAction *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  const char *getDescription() override;
 
   class ShowActionDescr : public ActionDescr
   {
@@ -114,14 +105,11 @@ public:
   RawShowAction( const RawShowAction &other );
   RawShowAction &operator=( const RawShowAction &other );
 
-  virtual RawShowAction *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual const char *getDescription();
+  RawShowAction *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  const char *getDescription() override;
 
   class RawShowActionDescr : public ActionDescr
   {
@@ -141,16 +129,13 @@ public:
   UserAction( const UserAction &other );
   UserAction &operator=( const UserAction &other );
 
-  virtual UserAction *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual const char *getDescription();
-  virtual bool save(Datei*);
-  virtual int configure();
+  UserAction *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  const char *getDescription() override;
+  bool save(Datei*) override;
+  int configure() override;
   int getNr() const;
   void setNr( int n );
 
@@ -176,16 +161,13 @@ public:
   ChangeHiddenFlag( const ChangeHiddenFlag &other );
   ChangeHiddenFlag &operator=( const ChangeHiddenFlag &other );
 
-  virtual ChangeHiddenFlag *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  ChangeHiddenFlag *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
   
   int getMode() const;
   void setMode( int m );
