@@ -1,6 +1,6 @@
 /* changesymlinkop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2015 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -64,7 +64,7 @@ ChangeSymlinkOp::getName()
 }
 
 int
-ChangeSymlinkOp::run( WPUContext *wpu, ActionMessage *msg )
+ChangeSymlinkOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
@@ -248,6 +248,15 @@ int ChangeSymlinkOp::configure()
                         endmode = 0;
                     } else if ( msg->button.button == cb ) endmode = 1;
                     break;
+                case AG_KEYPRESSED:
+                    if ( win->isParent( msg->key.window, false ) == true ) {
+                        switch ( msg->key.key ) {
+                            case XK_Escape:
+                                endmode = 1;
+                                break;
+                        }
+                    }
+                    break;
             }
             aguix->ReplyMessage( msg );
         }
