@@ -1,6 +1,6 @@
 /* chmodop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,6 +24,7 @@
 
 #include "wdefines.h"
 #include "functionproto.h"
+#include "chmodorder.hh"
 
 class ChModOp:public FunctionProto
 {
@@ -33,33 +34,36 @@ public:
   ChModOp( const ChModOp &other );
   ChModOp &operator=( const ChModOp &other );
 
-  virtual ChModOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  ChModOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
   
   int doconfigure(int mode);
   
   void setOnFiles(bool);
   void setOnDirs(bool);
   void setRecursive(bool);
-  void setRequestFlags(bool);
   bool isInteractiveRun() const;
   void setInteractiveRun();
+
+    void setApplyMode( enum chmod_apply_mode mode );
+
+    void setPermissions( mode_t permissions );
+    void setPermissions( const std::string &permissions );
 protected:
   static const char *name;
   // Infos to save
 
   bool onfiles,
        ondirs,
-       recursive,
-       requestflags;
+       recursive;
+
+    enum chmod_apply_mode m_apply_mode = CHMOD_ASK_PERMISSIONS;
+    mode_t m_permissions = 0;
   
   // temp variables
   Lister *startlister;
@@ -67,6 +71,9 @@ protected:
   bool tonfiles,
        tondirs,
        trecursive;
+
+    enum chmod_apply_mode m_tapply_mode;
+    mode_t m_tpermissions = 0;
   
   int normalmodechmod( ActionMessage* );
 };
