@@ -1,6 +1,6 @@
 /* chownop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -33,23 +33,19 @@ public:
   ChOwnOp( const ChOwnOp &other );
   ChOwnOp &operator=( const ChOwnOp &other );
 
-  virtual ChOwnOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  ChOwnOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
   
   int doconfigure(int mode);
   
   void setOnFiles(bool);
   void setOnDirs(bool);
   void setRecursive(bool);
-  void setRequestFlags(bool);
   bool isInteractiveRun() const;
   void setInteractiveRun();
 protected:
@@ -58,8 +54,7 @@ protected:
 
   bool onfiles,
        ondirs,
-       recursive,
-       requestflags;
+       recursive;
   
   // temp variables
   Lister *startlister;
