@@ -1,6 +1,6 @@
 /* chtimeop.c
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2015-2017 Ralf Hoffmann.
+ * Copyright (C) 2015-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -69,7 +69,7 @@ const char * ChTimeOp::getName()
     return name;
 }
 
-int ChTimeOp::run( WPUContext *wpu, ActionMessage *msg )
+int ChTimeOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     ListerMode *lm1;
 
