@@ -1,6 +1,6 @@
 /* configtokens.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2003-2022 Ralf Hoffmann.
+ * Copyright (C) 2003-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -191,6 +191,7 @@ struct mytoken tokenlist[]={
   {TERMINALWAIT_WCP,"terminalwait"},
   {SHOWOUTPUT_WCP,"showoutput"},
   {SHOWOUTPUTINT_WCP,"showoutputint"},
+  {SHOWOUTPUTOTHERSIDE_WCP,"showoutputotherside"},
   {VIEWSTR_WCP,"viewstr"},
   {INBACKGROUND_WCP,"inbackground"},
   {TAKEDIRS_WCP,"takedirs"},
@@ -246,6 +247,7 @@ struct mytoken tokenlist[]={
   {WINDOW_WCP,"window"},
   {GOTO_WCP,"goto"},
   {PUSHUSEOUTPUT_WCP,"pushuseoutput"},
+  {PUSHOUTPUTRETURNCODE_WCP,"pushoutputreturncode"},
   {DODEBUG_WCP,"dodebug"},
   {WPURECURSIVE_WCP,"wpurecursive"},
   {WPUTAKEDIRS_WCP,"wputakedirs"},
@@ -319,6 +321,7 @@ struct mytoken tokenlist[]={
   { SCROLLBUTTON_WCP, "scrollbutton" },
   { SELECTMETHOD_WCP, "selectmethod" },
   { ALTERNATIVE_WCP, "alternative" },
+  { DELAYEDCONTEXTMENU_WCP, "delayedcontextmenu" },
   { SEARCHOP_WCP, "searchop" },
   { EDITCOMMAND_WCP, "editcommand" },
   { SHOWPREVRESULTS_WCP, "showprevresults" },
@@ -469,7 +472,27 @@ struct mytoken tokenlist[]={
   { REMOVE_WCP, "remove" },
   { HIDENONEXISTING_WCP, "hidenonexisting" },
   { VIEWCOMMANDLOGOP_WCP, "viewcommandlogop" },
-  { LOCKONCURRENTDIR_WCP, "lockoncurrentdir" }
+  { LOCKONCURRENTDIR_WCP, "lockoncurrentdir" },
+  { STARTNODE_WCP, "startnode" },
+  { MENUS_WCP, "menus" },
+  { LVMODES_WCP, "lvmodes" },
+  { LVCOMMANDS_WCP, "lvcommands" },
+  { TOPLEVEL_WCP, "toplevel" },
+  { FOLLOWACTIVE_WCP, "followactive" },
+  { WATCHFILE_WCP, "watchfile" },
+  { ACTIVATETEXTVIEWMODEOP_WCP, "activatetextviewmodeop" },
+  { SOURCEMODE_WCP, "sourcemode" },
+  { ACTIVEFILE_WCP, "activefile" },
+  { FILETYPEACTION_WCP, "filetypeaction" },
+  { IGNOREDIRS2_WCP, "ignoredirs2" },
+  { IGNOREDIR_WCP, "ignoredir" },
+  { CUSTOMATTRIBUTE_WCP, "customattribute" },
+  { SHOWOUTPUTCUSTOMATTRIBUTE_WCP, "showoutputcustomattribute" },
+  { STARTSEARCHSTRING_WCP, "startsearchstring" },
+  { APPLYONLY_WCP, "applyonly" },
+  { USESELECTEDENTRIES_WCP, "useselectedentries" },
+  { ONLYEXACTPATH_WCP, "onlyexactpath" },
+  { ALSOSELECTEXTENSION_WCP, "alsoselectextension" }
 };
 
 ArrayList *tokenlist_sorted = NULL;
