@@ -1,6 +1,6 @@
 /* copyop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -34,14 +34,13 @@ public:
   CopyOp( const CopyOp &other );
   CopyOp &operator=( const CopyOp &other );
 
-  virtual CopyOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-  using FunctionProto::run;
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  CopyOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
 
   int doconfigure(int mode);
 
@@ -51,7 +50,6 @@ public:
   void setRename(bool);
   void setSameDir(bool);
   void setRequestDest(bool);
-  void setRequestFlags(bool);
   void setOverwrite(overwrite_t);
   void setPreserveAttr(bool);
   void setVdirPreserveDirStructure(bool);
@@ -77,7 +75,6 @@ protected:
   bool vdir_preserve_dir_structure;
   adjust_relative_symlinks_t adjust_relative_symlinks;
   CopyParams::EnsureFilePermissions ensure_file_permissions;
-  bool request_flags;
 
   overwrite_t overwrite;
 
