@@ -1,6 +1,6 @@
 /* createsymlinkop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -33,29 +33,25 @@ public:
   CreateSymlinkOp( const CreateSymlinkOp &other );
   CreateSymlinkOp &operator=( const CreateSymlinkOp &other );
 
-  virtual CreateSymlinkOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  CreateSymlinkOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
   
   int doconfigure(int mode);
   
   void setSameDir(bool);
   void setLocal(bool);
-  void setRequestFlags(bool);
   bool isInteractiveRun() const;
   void setInteractiveRun();
 protected:
   static const char *name;
   // Infos to save
 
-  bool same_dir,local,requestflags;
+  bool same_dir,local;
   
   // temp variables
   Lister *startlister,*endlister;
