@@ -1,6 +1,6 @@
 /* datei.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2022 Ralf Hoffmann.
+ * Copyright (C) 2001-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -81,10 +81,10 @@ public:
   static d_fe_t lfileExistsExt(const char*);
   static char *getFilenameFromPath(const char*);
   static char *getNameWithoutExt(const char*);
+    static const char *getExtension( const char *filename );
     static char *createTMPName( const char *infix = NULL,
                                 const char *suffix = NULL );
   static std::string createTMPHardlink( const std::string &name );
-  static char *shrinkFilename( const char *str, int maxlength, AWidth &lencalc );
   static loff_t fileSize( const char * );
   static char *getRelativePath( const char *source, const char *destdir, bool use_realpath );
 
