@@ -20,11 +20,11 @@
  */
 
 #include "dirbookmarkaddui.hh"
-#include <aguix/aguix.h>
-#include <aguix/awindow.h>
-#include <aguix/text.h>
-#include <aguix/button.h>
-#include <aguix/acontainerbb.h>
+#include "aguix/aguix.h"
+#include "aguix/awindow.h"
+#include "aguix/text.h"
+#include "aguix/button.h"
+#include "aguix/acontainerbb.h"
 #include "worker_locale.h"
 #include "bookmarkdbentry.hh"
 #include "nwc_path.hh"
