@@ -840,7 +840,7 @@ DirCompareOp::doConfigure()
     ac0->setBorderWidth( 5 );
 
     RefCount<AFontWidth> lencalc( new AFontWidth( aguix, NULL ) );
-    TextStorageString help_ts( catalog.getLocale( 1180 ), lencalc );
+    auto help_ts = std::make_shared< TextStorageString >( catalog.getLocale( 1180 ), lencalc );
     TextView *help_tv = (TextView*)ac0->add( new TextView( aguix,
                                                            0, 0, 400, 50, "", help_ts ),
                                              0, 0, AContainer::CO_MIN );
