@@ -1,6 +1,6 @@
 /* enterpathop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -63,7 +63,7 @@ EnterPathOp::getName()
 }
 
 int
-EnterPathOp::run( WPUContext *wpu, ActionMessage *msg )
+EnterPathOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     if ( msg->mode != msg->AM_MODE_DNDACTION ) {
         startlister = msg->getWorker()->getActiveLister();
@@ -217,6 +217,15 @@ EnterPathOp::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
