@@ -1,6 +1,6 @@
 /* externalvdirop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2017-2021 Ralf Hoffmann.
+ * Copyright (C) 2017-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -31,23 +31,40 @@
 #include "aguix/awindow.h"
 #include "aguix/textstorage.h"
 #include "aguix/textview.h"
+#include "aguix/choosebutton.h"
+#include "aguix/fieldlistview.h"
 #include "virtualdirmode.hh"
 #include "nwc_path.hh"
+#include "workerinitialsettings.hh"
+#include "pers_string_list.hh"
 
 int ExternalVDirOp::s_vdir_number = 1;
 
 const char *ExternalVDirOp::name = "ExternalVDirOp";
 
+int ExternalVDirOp::instance_counter = 0;
+size_t ExternalVDirOp::history_size = 1000;
+std::unique_ptr< PersistentStringList > ExternalVDirOp::m_history;
+
 ExternalVDirOp::ExternalVDirOp() : FunctionProto()
 {
     m_command_string = "";
     m_category = FunctionProto::CAT_FILELIST;
 
+    if ( instance_counter == 0 ) {
+        initHistory();
+    }
+    instance_counter++;
+
     hasConfigure = true;
 }
 
 ExternalVDirOp::~ExternalVDirOp()
 {
+    instance_counter--;
+    if ( instance_counter == 0 ) {
+        closeHistory();
+    }
 }
 
 ExternalVDirOp*
@@ -55,6 +72,7 @@ ExternalVDirOp::duplicate() const
 {
     ExternalVDirOp *ta = new ExternalVDirOp();
     ta->setCommandString( m_command_string );
+    ta->setRequestParameters( requestParameters() );
     return ta;
 }
 
@@ -135,7 +153,13 @@ ExternalVDirOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
     std::string res_str;
     int res = 0;
 
-    if ( wpu->parse( m_command_string.c_str(),
+    if ( requestParameters() == true ) {
+        if ( doconfigure( 1 ) != 0 ) return 0;
+    } else {
+        m_t_command_string = m_command_string;
+    }
+  
+    if ( wpu->parse( m_t_command_string.c_str(),
                      res_str,
                      a_max( EXE_STRING_LEN - 1024, 256 ),
                      true, WPUContext::PERSIST_FLAGS ) == WPUContext::PARSE_SUCCESS ) {
@@ -164,7 +188,7 @@ ExternalVDirOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
                 str1 += error;
 
                 RefCount<AFontWidth> lencalc( new AFontWidth( Worker::getAGUIX(), NULL ) );
-                TextStorageString ts( str1, lencalc );
+                auto ts = std::make_shared< TextStorageString >( str1, lencalc );
 
                 Worker::getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
             }
@@ -180,6 +204,11 @@ ExternalVDirOp::save(Datei *fh)
 {
     if ( fh == NULL ) return false;
     fh->configPutPairString( "commandstring", m_command_string.c_str() );
+
+    if ( requestParameters() ) {
+        fh->configPutPairBool( "requestflags", requestParameters() );
+    }
+
     return true;
 }
 
@@ -195,6 +224,24 @@ ExternalVDirOp::configure()
     return doconfigure(0);
 }
 
+
+static void show_matching_entries( const std::list< std::string > &list,
+                                   const std::string &prefix,
+                                   FieldListView *lv )
+{
+    lv->setSize( 0 );
+
+    for ( auto &p : list ) {
+        if ( ! AGUIXUtils::starts_with( p, prefix ) ) continue;
+
+        int row = lv->insertRow( 0 );
+        lv->setText( row, 0, p );
+        lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+    }
+
+    lv->redraw();
+}
+
 int
 ExternalVDirOp::doconfigure( int mode )
 {
@@ -209,14 +256,14 @@ ExternalVDirOp::doconfigure( int mode )
     win = new AWindow( aguix, 10, 10, 10, 10, title.c_str(), AWindow::AWINDOW_DIALOG );
     win->create();
 
-    AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
+    AContainer *ac1 = win->setContainer( new AContainer( win, 1, 5 ), true );
     ac1->setMinSpace( 5 );
     ac1->setMaxSpace( 5 );
 
     RefCount<AFontWidth> lencalc( new AFontWidth( aguix, NULL ) );
-    auto help_ts = std::unique_ptr< TextStorageString >( new TextStorageString( catalog.getLocale( 1206 ), lencalc ) );
+    auto help_ts = std::shared_ptr< TextStorageString >( new TextStorageString( catalog.getLocale( 1206 ), lencalc ) );
     TextView *help_tv = ac1->addWidget( new TextView( aguix,
-                                                      0, 0, 300, 80, "", *help_ts ),
+                                                      0, 0, 300, 80, "", help_ts ),
                                         0, 0, AContainer::CO_INCW );
     help_tv->setLineWrap( true );
     help_tv->maximizeYLines( 10, 500 );
@@ -242,8 +289,28 @@ ExternalVDirOp::doconfigure( int mode )
                                                   0,
                                                   "F",
                                                   0 ), 2, 0, AContainer::CO_FIX );
-  
-    AContainer *ac1_5 = ac1->add( new AContainer( win, 2, 1 ), 0, 2 );
+    auto *history_lv = ac1->addWidget( new FieldListView( aguix, 0, 0, 100, 7 * aguix->getCharHeight(), 0 ), 0, 2, AContainer::CO_MIN );
+
+    history_lv->setNrOfFields( 1 );
+    history_lv->setGlobalFieldSpace( 5 );
+    history_lv->setFieldText( 0, catalog.getLocale( 1565 ) );
+    history_lv->setShowHeader( true );
+    history_lv->setConsiderHeaderWidthForDynamicWidth( true );
+    history_lv->setHBarState( 2 );
+    history_lv->setVBarState( 2 );
+    history_lv->setDisplayFocus( true );
+    history_lv->setAcceptFocus( true );
+
+    std::list< std::string > cur_history = m_history->getList();
+    show_matching_entries( cur_history, m_command_string, history_lv );
+
+    ChooseButton *rfcb = nullptr;
+    if ( mode == 0 ) {
+        rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( requestParameters() == true ) ? 1 : 0,
+                                                          catalog.getLocale( 294 ), LABEL_RIGHT, 0 ), 0, 3, AContainer::CO_INCWNR );
+    }
+
+    AContainer *ac1_5 = ac1->add( new AContainer( win, 2, 1 ), 0, 4 );
     ac1_5->setMinSpace( 5 );
     ac1_5->setMaxSpace( -1 );
     ac1_5->setBorderWidth( 0 );
@@ -261,6 +328,8 @@ ExternalVDirOp::doconfigure( int mode )
 
     win->setDoTabCycling( true );
     win->show();
+    sgcf->takeFocus();
+
     for( ; endmode == -1; ) {
         msg = aguix->WaitMessage( win );
         if ( msg != NULL ) {
@@ -292,9 +361,30 @@ ExternalVDirOp::doconfigure( int mode )
                             case XK_Escape:
                                 endmode = 1;
                                 break;
+                            case XK_Down:
+                                if ( sgcf->isActive() ) {
+                                    history_lv->setActiveRow( 0 );
+                                    history_lv->takeFocus();
+                                    history_lv->showActive();
+                                    sgcf->setText( history_lv->getText( 0, 0 ).c_str() );
+                                }
+                                break;
                         }
                     }
                     break;
+                case AG_STRINGGADGET_CONTENTCHANGE:
+                    if ( msg->stringgadget.sg == sgcf ) {
+                        show_matching_entries( cur_history, sgcf->getText(), history_lv );
+                    }
+                    break;
+                case AG_FIELDLV_ONESELECT:
+                case AG_FIELDLV_MULTISELECT: {
+                    int row = history_lv->getActiveRow();
+                    if ( history_lv->isValidRow( row ) == true ) {
+                        sgcf->setText( history_lv->getText( row, 0 ).c_str() );
+                    }
+                    break;
+                }
             }
             aguix->ReplyMessage( msg );
         }
@@ -302,7 +392,14 @@ ExternalVDirOp::doconfigure( int mode )
   
     if ( endmode == 0 ) {
         // ok
-        setCommandString( sgcf->getText() );
+
+        if ( mode == 1 ) {
+            m_t_command_string = sgcf->getText();
+            addHistoryItem( m_t_command_string );
+        } else {
+            setCommandString( sgcf->getText() );
+            setRequestParameters( rfcb->getState() );
+        }
     }
     delete win;
 
@@ -313,3 +410,59 @@ void ExternalVDirOp::setCommandString( const std::string &str )
 {
     m_command_string = str;
 }
+
+bool ExternalVDirOp::isInteractiveRun() const
+{
+    return true;
+}
+
+void ExternalVDirOp::setInteractiveRun()
+{
+    setRequestParameters( true );
+}
+
+void ExternalVDirOp::initHistory()
+{
+    std::string str1;
+
+    str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
+    str1 = NWC::Path::join( str1, "externalvdir-history" );
+
+    m_history = std::unique_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
+}
+
+void ExternalVDirOp::closeHistory()
+{
+    m_history.reset();
+}
+
+std::list< std::string > ExternalVDirOp::getHistory()
+{
+    if ( m_history ) {
+        return m_history->getList();
+    }
+
+    std::string str1;
+
+    str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
+    str1 = NWC::Path::join( str1, "externalvdir-history" );
+
+    auto h = std::unique_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
+
+    return h->getList();
+}
+
+void ExternalVDirOp::addHistoryItem( const std::string &str )
+{
+    if ( str.empty() ) return;
+
+    if ( m_history->contains( str ) ) {
+        m_history->removeEntry( str );
+        m_history->pushFrontEntry( str );
+    } else {
+        if ( m_history->size() >= history_size ) {
+            m_history->removeLast();
+        }
+        m_history->pushFrontEntry( str );
+    }
+}
