@@ -155,7 +155,7 @@ void FileEntryColor::setCustomColor( const FileEntryCustomColor &c )
     m_custom_color.reset( new FileEntryCustomColor( c ) );
 }
 
-const FileEntryCustomColor &FileEntryColor::getCustomColor() const throw ( int )
+const FileEntryCustomColor &FileEntryColor::getCustomColor() const
 {
     if ( m_custom_color.get() == NULL ) throw 1;
     return *m_custom_color;
@@ -166,7 +166,7 @@ void FileEntryColor::setOverrideColor( const FileEntryCustomColor &c )
     m_override_color.reset( new FileEntryCustomColor( c ) );
 }
 
-const FileEntryCustomColor &FileEntryColor::getOverrideColor() const throw ( int )
+const FileEntryCustomColor &FileEntryColor::getOverrideColor() const
 {
     if ( m_override_color.get() == NULL ) throw 1;
     return *m_override_color;
