@@ -21,7 +21,7 @@
 
 #include "fileentry_typecheck.hh"
 #include "fileentry.hh"
-#include <aguix/lowlevelfunc.h>
+#include "aguix/lowlevelfunc.h"
 #include "pdatei.h"
 #include "condparser.h"
 #include "stringbuf.h"
@@ -74,7 +74,7 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::runTests( FileEntry &fe,
                             }
                         }
                         if ( y == 0 ) {
-                            erg.prio = 3;
+                            erg.accuracy = 3;
                             erg.type = type;
                         }
                     }
@@ -107,7 +107,7 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::runTests( FileEntry &fe,
                 }
                 
                 if ( type->magicMatch( temp_results.magic_result[array_pos] ) == true ) {
-                    erg.prio = 3;
+                    erg.accuracy = 3;
                     erg.type = type;
                 }
             }
@@ -121,7 +121,7 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::runTests( FileEntry &fe,
                 erg.tests_done++;
                 condparser->setIgnoreContent( skipcontent );
                 if ( condparser->parse( type->getExtCond(), &fe ) == 1 ) {
-                    erg.prio = 2;
+                    erg.accuracy = 2;
                     erg.type = type;
                 }
             }
@@ -139,12 +139,17 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::runTests( FileEntry &fe,
                     tstr1 = fe.name;
                 }
                 if ( type->patternMatchString( tstr1 ) == true ) {
-                    erg.prio = 1;
+                    erg.accuracy = 1;
                     erg.type = type;
                 }
             }
         }
     }
+
+    if ( erg.type ) {
+        erg.priority = type->priority();
+    }
+
     return erg;
 }
 
@@ -157,7 +162,7 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::checkFiletype( FileEntry
                                                                        typecheck_temp_results_t &temp_results )
 {
     WCFiletype *ft = NULL;
-    typecandidate_t cand1, erg, cand2;
+    typecandidate_t erg;
     const std::list<WCFiletype*> *subtypes = NULL;
     std::list<WCFiletype*>::const_iterator it1;
 
@@ -179,25 +184,34 @@ FileEntryTypeCheck::typecandidate_t FileEntryTypeCheck::checkFiletype( FileEntry
 
     for ( it1 = ftlist->begin(); it1 != ftlist->end(); it1++ ) {
         ft = *it1;
-        cand1 = runTests( fe, ft, skipcontent, condparser, fbuffer, reads, temp_results );
+        typecandidate_t cand1 = runTests( fe, ft, skipcontent, condparser, fbuffer, reads, temp_results );
         if ( ( cand1.type != NULL ) || ( cand1.tests_done < 1 ) ) {
             // check subtypes if we had a hit or did no tests
             subtypes = ft->getSubTypeList();
             if ( subtypes != NULL ) {
                 if ( subtypes->size() > 0 ) {
-                    cand2 = checkFiletype( fe, subtypes, skipcontent, condparser, fbuffer, reads, temp_results );
-                    if ( cand2.prio > 0 ) {
+                    typecandidate_t  cand2 = checkFiletype( fe, subtypes, skipcontent, condparser, fbuffer, reads, temp_results );
+
+                    // don't check against priority of parent since
+                    // children will always win, or otherwise would
+                    // never win
+                    if ( cand2.accuracy > 0 ) {
                         // subtypes are always more important
-                        cand1.prio = cand2.prio;
+                        cand1.accuracy = cand2.accuracy;
                         cand1.type = cand2.type;
+                        cand1.priority = ft->priority() + cand2.priority;
                     }
                 }
             }
         }
         if ( cand1.type != NULL ) {
-            if ( cand1.prio > erg.prio ) {
-                erg.prio = cand1.prio;
+            if ( erg.type == NULL ||
+                 cand1.priority < erg.priority ||
+                 ( cand1.priority == erg.priority &&
+                   cand1.accuracy > erg.accuracy ) ) {
+                erg.accuracy = cand1.accuracy;
                 erg.type = cand1.type;
+                erg.priority = cand1.priority;
             }
         }
     }
@@ -213,7 +227,7 @@ WCFiletype* FileEntryTypeCheck::checkFiletype( FileEntry &fe,
     int id;
     WCFiletype *ft = NULL, *unknownft = NULL;
     CondParser *localparser = NULL;
-    typecandidate_t cand1, erg, cand2;
+    typecandidate_t erg;
     const std::list<WCFiletype*> *subtypes = NULL;
     typecheck_temp_results_t temp_results;
 
@@ -266,25 +280,34 @@ WCFiletype* FileEntryTypeCheck::checkFiletype( FileEntry &fe,
     id = ftlist->initEnum();
     ft = (WCFiletype*)ftlist->getFirstElement( id );
     while ( ft != NULL ) {
-        cand1 = runTests( fe, ft, skipcontent, condparser, fbuffer, reads, temp_results );
+        typecandidate_t cand1 = runTests( fe, ft, skipcontent, condparser, fbuffer, reads, temp_results );
         if ( ( cand1.type != NULL ) || ( cand1.tests_done < 1 ) ) {
             // check subtypes if we had a hit or did no tests
             subtypes = ft->getSubTypeList();
             if ( subtypes != NULL ) {
                 if ( subtypes->size() > 0 ) {
-                    cand2 = checkFiletype( fe, subtypes, skipcontent, condparser, fbuffer, reads, temp_results );
-                    if ( cand2.prio > 0 ) {
+                    typecandidate_t cand2 = checkFiletype( fe, subtypes, skipcontent, condparser, fbuffer, reads, temp_results );
+
+                    // don't check against priority of parent since
+                    // children will always win, or otherwise would
+                    // never win
+                    if ( cand2.accuracy > 0 ) {
                         // subtypes are always more important
-                        cand1.prio = cand2.prio;
+                        cand1.accuracy = cand2.accuracy;
                         cand1.type = cand2.type;
+                        cand1.priority = ft->priority() + cand2.priority;
                     }
                 }
             }
         }
         if ( cand1.type != NULL ) {
-            if ( cand1.prio > erg.prio ) {
-                erg.prio = cand1.prio;
+            if ( erg.type == NULL ||
+                 cand1.priority < erg.priority ||
+                 ( cand1.priority == erg.priority &&
+                   cand1.accuracy > erg.accuracy ) ) {
+                erg.accuracy = cand1.accuracy;
                 erg.type = cand1.type;
+                erg.priority = cand1.priority;
             }
         }
         ft = (WCFiletype*)ftlist->getNextElement( id );
