@@ -22,13 +22,18 @@
 #include "getmntinfomounttable.hh"
 #include "mounttable_entry.hh"
 #include "nwc_fsentry.hh"
-#ifdef HAVE_GETMNTINFO
+#ifdef HAVE_GETMNTINFO_FBSD
 #include <sys/param.h>
 #include <sys/ucred.h>
 #include <sys/mount.h>
 #endif
 
-#ifdef HAVE_GETMNTINFO
+#ifdef HAVE_GETMNTINFO_NBSD
+#include <sys/types.h>
+#include <sys/statvfs.h>
+#endif
+
+#if defined( HAVE_GETMNTINFO_FBSD ) || defined( HAVE_GETMNTINFO_NBSD )
 typedef std::pair< mode_t, dev_t > w_osdevice_t;
 
 static w_osdevice_t getOSDevice( const std::string &dev )
@@ -57,11 +62,20 @@ std::list<MountTableEntry> GetMntInfoMountTable::readEntries()
 {
     std::list<MountTableEntry> l;
 
-#ifdef HAVE_GETMNTINFO
-    struct statfs *mount_entries;
-    int n;
+#if defined( HAVE_GETMNTINFO_FBSD ) || defined( HAVE_GETMNTINFO_NBSD )
 
+#if defined( HAVE_GETMNTINFO_FBSD )
+    struct statfs *mount_entries;
+#elif defined( HAVE_GETMNTINFO_NBSD )
+    struct statvfs *mount_entries;
+#endif
+    int n = 0;
+
+#if defined( HAVE_GETMNTINFO_FBSD )
     n = getmntinfo( &mount_entries, MNT_NOWAIT );
+#elif defined( HAVE_GETMNTINFO_NBSD )
+    n = getmntinfo( &mount_entries, ST_NOWAIT );
+#endif
     for ( int i = 0; i < n; i++ ) {
         if ( mount_entries[i].f_mntfromname != NULL &&
              mount_entries[i].f_mntonname != NULL ) {
