@@ -199,8 +199,8 @@ HintDB::HintDB( const std::string &file,
                 WConfig &wconf ) : m_file( file ),
                                    m_wconfig( wconf )
 {
-    m_flag_producer = new HintDBFlagProducer( m_wconfig );
-    m_flag_replacer = new FlagReplacer( *m_flag_producer );
+    m_flag_producer = std::make_shared< HintDBFlagProducer >( m_wconfig );
+    m_flag_replacer = std::make_shared< FlagReplacer >( m_flag_producer );
 
     read_db();
 }
