@@ -1,6 +1,6 @@
 /* listermode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2021 Ralf Hoffmann.
+ * Copyright (C) 2001-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -149,7 +149,7 @@ void ListerMode::registerCommand( const std::string &command,
 
     if ( ! description.empty() &&
          m_command_info.count( std::make_pair( command, false ) ) == 0 ) {
-        m_command_info[ std::make_pair( command, false ) ] = std::make_tuple( description, category );
+        m_command_info[ std::make_pair( command, false ) ] = std::make_tuple( description, category, nullptr );
     }
 }
 
@@ -171,13 +171,14 @@ void ListerMode::runCommand( const std::string &command, const std::list< RefCou
 void ListerMode::registerCommand( const std::string &command,
                                   const std::function< void ( const std::list< RefCount< ArgClass > > ) > callback,
                                   const std::string &description,
-                                  FunctionProto::command_categories_t category )
+                                  FunctionProto::command_categories_t category,
+                                  const std::function< std::string ( const std::string &current_input ) > help_callback )
 {
     m_commands_with_args[command] = callback;
 
     if ( ! description.empty() &&
          m_command_info.count( std::make_pair( command, true ) ) == 0 ) {
-        m_command_info[ std::make_pair( command, true ) ] = std::make_tuple( description, category );
+        m_command_info[ std::make_pair( command, true ) ] = std::make_tuple( description, category, help_callback );
     }
 }
 
@@ -314,6 +315,28 @@ WorkerTypes::reg_command_info_t ListerMode::getCommandInfo( const std::string &c
     return res;
 }
 
+std::string ListerMode::getCommandInputHelp( const std::string &command,
+                                             const std::string &current_input ) const
+{
+    auto f = std::get<2>( m_command_info.at( std::make_pair( command, true ) ) );
+    if ( f ) {
+        return f( current_input );
+    }
+    return "";
+}
+
+bool ListerMode::commandHasInputHelp( const std::string &command ) const
+{
+    if ( std::get<2>( m_command_info.at( std::make_pair( command, true ) ) ) ) {
+        return true;
+    }
+    return false;
+}
+
 void ListerMode::finalizeBGOps()
 {
 }
+
+void ListerMode::interpretFinalize()
+{
+}
