@@ -1,6 +1,6 @@
 /* menutree.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2013-2015 Ralf Hoffmann.
+ * Copyright (C) 2013-2015,2017 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -36,7 +36,7 @@ int MenuTree::setFilter( const std::string &filter )
     m_filter = filter;
     m_matcher.setMatchString( filter );
 
-    return checkVisibility( m_current_node );
+    return checkVisibility( "", m_current_node );
 }
 
 std::string MenuTree::createPath( const std::string &separator )
@@ -77,6 +77,26 @@ std::string MenuTree::createPath( MenuTreeNode *me, MenuTreeNode *node,
     return "";
 }
 
+std::string MenuTree::createFullname( MenuTreeNode *node,
+                                      const std::string &separator )
+{
+    if ( ! node ) return "";
+
+    if ( ! node->getParent() ) {
+        return "";
+    }
+
+    std::string res;
+
+    res = createFullname( node->getParent(), separator );
+    if ( ! res.empty() ) {
+        res += separator;
+    }
+    res += node->getName();
+
+    return res;
+}
+
 void MenuTree::setCurrentNode( MenuTreeNode *node )
 {
     m_current_node = node;
@@ -115,33 +135,63 @@ void MenuTree::fillFullname( MenuTreeNode *node,
     }
 }
 
-int MenuTree::checkVisibility( MenuTreeNode *node,
+int MenuTree::checkVisibility( const std::string &prefix,
+                               MenuTreeNode *node,
                                bool root )
 {
     if ( root ) node->clearMatchedFlag();
 
     int visible_elements = 0;
 
-    if ( root && m_filter.empty() ) {
+    if ( m_visible_list.get() == NULL && root && m_filter.empty() ) {
         for ( const auto it1 : *node ) {
             it1->setMatched( true );
             visible_elements++;
         }
     }
 
-    if ( ! m_filter.empty() ) {
+    if ( ! m_filter.empty() || m_visible_list.get() != NULL ) {
         for ( const auto it1 : *node ) {
-            if ( m_matcher.match( it1->getName() ) ) {
+            bool matched = false;
+            std::string fullname = prefix;
+            if ( ! fullname.empty() ) fullname += "/";
+            fullname += it1->getName();
+
+            int visible_pos = -1;
+
+            if ( m_visible_list.get() ) {
+                visible_pos = m_visible_list->findPosition( createFullname( it1, "/" ) );
+                if ( visible_pos >= 0 ) {
+                    matched = true;
+                }
+            } else {
+                matched = true;
+            }
+
+            if ( matched ) {
+                matched =m_matcher.match( it1->getName() );
+            }
+            
+            if ( matched ) {
                 it1->setMatched( true );
+                it1->setVisiblePos( visible_pos );
                 visible_elements++;
                 
-                visible_elements += checkVisibility( it1, false );
+                visible_elements += checkVisibility( fullname, it1, false );
             } else {
                 it1->setMatched( false );
-                visible_elements += checkVisibility( it1, false );
+                it1->setVisiblePos( -1 );
+                visible_elements += checkVisibility( fullname, it1, false );
             }
         }
     }
 
     return visible_elements;
 }
+
+int MenuTree::setVisibleList( std::shared_ptr< PersistentStringList > list )
+{
+    m_visible_list = list;
+
+    return 0;
+}
