@@ -1,6 +1,6 @@
 /* nmextlist.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -35,7 +35,6 @@
 NMExtList::NMExtList()
 {
   extlist = new List();
-  files = dirs = 0;
   take_dirs = false;
 }
 
@@ -52,16 +51,6 @@ NMExtList::~NMExtList()
   delete extlist;
 }
 
-int NMExtList::getSize()
-{
-  return extlist->size();
-}
-
-int NMExtList::getRealSize()
-{
-  return ( files + ( ( take_dirs == true ) ? dirs : 0 ) );
-}
-
 void NMExtList::setTakeDirs( bool nv )
 {
   take_dirs = nv;
@@ -79,20 +68,51 @@ void NMExtList::closeEnum( int id )
 
 NM_extern_fe *NMExtList::getFirstElement( int id )
 {
-  return (NM_extern_fe*)extlist->getFirstElement( id );
+    if ( m_list_done ) {
+        NM_extern_fe *e = (NM_extern_fe*)extlist->getFirstElement( id );
+        if ( e ) {
+            m_first_fullname = e->getFullname( false );
+        } else {
+            m_first_fullname.clear();
+        }
+        return e;
+    }
+
+    if ( ! m_ondemand_mode ) return NULL;
+
+    if ( extlist->size() == 0 ) {
+        gatherNextElement();
+    }
+
+    NM_extern_fe *e = (NM_extern_fe*)extlist->getFirstElement( id );
+    if ( e ) {
+        m_first_fullname = e->getFullname( false );
+    } else {
+        m_first_fullname.clear();
+    }
+    return e;
 }
 
 NM_extern_fe *NMExtList::getNextElement( int id )
 {
-  return (NM_extern_fe*)extlist->getNextElement( id );
+    if ( m_list_done ) {
+        return (NM_extern_fe*)extlist->getNextElement( id );
+    }
+
+    if ( ! m_ondemand_mode ) return NULL;
+
+    if ( ! extlist->hasNextElement( id ) ) {
+        gatherNextElement();
+    }
+
+    return (NM_extern_fe*)extlist->getNextElement( id );
 }
 
 int NMExtList::removeElement( NM_extern_fe *efe )
 {
   if ( efe == NULL ) return 1;
-  if ( efe->getDirFinished() != 0 ) dirs--;
-  else files--;
   extlist->removeElement( efe );
+  m_list_modified = true;
   return 0;
 }
 
@@ -106,8 +126,7 @@ void NMExtList::createExtList( ListerMode *lm, bool rec )
 
   createExtList( splist, rec );
 
-  files = getEntriesRealSize();
-  dirs = extlist->size() - files;
+  m_list_done = true;
 }
 
 class ExtListDirWalk : public NWC::DirWalkCallBack
@@ -170,6 +189,9 @@ void NMExtList::createExtList( ListerMode *lm, ActionMessage *msg, bool rec )
   std::unique_ptr<NM_specialsourceExt> ss1;
   const DNDMsg *dm;
 
+  // mark as done for now, undo if necessary
+  m_list_done = true;
+
   switch ( msg->mode ) {
     case ActionMessage::AM_MODE_SPECIAL:
         if ( msg->getFE() ) {
@@ -199,6 +221,14 @@ void NMExtList::createExtList( ListerMode *lm, ActionMessage *msg, bool rec )
             }
         }
       break;
+    case ActionMessage::AM_MODE_ON_DEMAND:
+        m_ondemand_mode = true;
+        m_ondemand_lm = lm;
+        m_ondemand_rec = rec;
+
+        // in on-demand mode, the list is not done yet
+        m_list_done = false;
+        break;
     case ActionMessage::AM_MODE_ONLYACTIVE:
       break;
     default:
@@ -206,9 +236,6 @@ void NMExtList::createExtList( ListerMode *lm, ActionMessage *msg, bool rec )
   }
 
   createExtList( splist, rec );
-
-  files = getEntriesRealSize();
-  dirs = extlist->size() - files;
 }
 
 int NMExtList::create_externfe_rec( const char *fullname, bool rec, const FileEntry *fe )
@@ -253,16 +280,72 @@ int NMExtList::createExtList( std::list< NM_specialsourceExt > &splist, bool rec
     return 0;
 }
 
-int NMExtList::getEntriesRealSize()
+void NMExtList::resetModifedFlag()
 {
-  int id = extlist->initEnum(),rs;
-  NM_extern_fe *efe1;
-  efe1=(NM_extern_fe*)extlist->getFirstElement(id);
-  rs=0;
-  while(efe1!=NULL) {
-    if(efe1->getDirFinished()==0) rs++;
-    efe1=(NM_extern_fe*)extlist->getNextElement(id);
-  }
-  extlist->closeEnum(id);
-  return rs;
+    m_list_modified = false;
+}
+
+bool NMExtList::modified() const
+{
+    return m_list_modified;
+}
+
+bool NMExtList::empty()
+{
+    if ( extlist->size() != 0 ) return false;
+
+    if ( m_list_done ) return true;
+
+    gatherNextElement();
+
+    return extlist->size() == 0;
+}
+
+void NMExtList::gatherNextElement()
+{
+    if ( ! m_ondemand_mode ) return;
+    if ( m_list_done ) return;
+
+    if ( ! m_ondemand_lm ) return;
+
+    if ( m_ondemand_state_counter == 0 ) {
+        // get active
+        std::list< NM_specialsourceExt > splist;
+
+        m_ondemand_lm->getSelFiles( splist, ListerMode::LM_GETFILES_ONLYACTIVE );
+
+        for ( auto &ss1 : splist ) {
+            if ( ss1.entry() ) {
+                create_externfe_rec( ss1.entry()->fullname,
+                                     m_ondemand_rec,
+                                     ss1.entry() );
+                m_ondemand_active_entry = ss1.entry()->fullname;
+            }
+        }
+
+    } else {
+        // get rest, filter active
+        std::list< NM_specialsourceExt > splist;
+
+        m_ondemand_lm->getSelFiles( splist, ListerMode::LM_GETFILES_ONLYSELECT );
+
+        for ( auto &ss1 : splist ) {
+            if ( ss1.entry() ) {
+                if ( m_ondemand_active_entry != ss1.entry()->fullname ) {
+                    create_externfe_rec( ss1.entry()->fullname,
+                                         m_ondemand_rec,
+                                         ss1.entry() );
+                }
+            }
+        }
+
+        m_list_done = true;
+    }
+
+    m_ondemand_state_counter++;
+}
+
+const std::string &NMExtList::getFirstFullname() const
+{
+    return m_first_fullname;
 }
