@@ -82,7 +82,7 @@ ParentActionOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
       amsg2.filetype = ft;
       amsg2.m_action_descr = msg->m_action_descr;
 
-      if ( msg->m_action_descr.getVal() != NULL ) {
+      if ( msg->m_action_descr.get() != NULL ) {
           l1 = ft->getActionList( false, *(msg->m_action_descr) );
       }
 
