@@ -1,6 +1,6 @@
 /* pdatei.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2019 Ralf Hoffmann.
+ * Copyright (C) 2002-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -41,7 +41,7 @@ class PDatei
   } open_param_t;
 
   int open( const char *filename, open_param_t mode = NORMAL_OPEN );
-  int read( void *buf, int size );
+  ssize_t read( void *buf, size_t size );
   void close();
   static enum pdatei_handler getCruncher( const char *filename );
   enum pdatei_handler getHandler();
