@@ -603,6 +603,26 @@ std::map< std::string, DeepPathStore > PersDeepPathStore::getKeywordCopy()
 
 loff_t PersDeepPathStore::getLastSize() const
 {
+    if ( m_lastsize == 0 ) {
+        std::string fn = m_main_db_file;
+
+        NWC::FSEntry tfe( fn );
+
+        if ( ! tfe.entryExists() ) {
+            fn = m_old_main_db_file;
+        }
+
+        NWC::FSEntry fe( fn );
+
+        if ( ! fe.entryExists() ) return 0;
+
+        if ( fe.isLink() ) {
+            return fe.stat_dest_size();
+        } else {
+            return fe.stat_size();
+        }
+    }
+
     return m_lastsize;
 }
 
