@@ -20,7 +20,7 @@
  */
 
 #include "prefixpathtree.hh"
-#include <aguix/util.h>
+#include "aguix/util.h"
 
 PrefixPathTree::PrefixPathTree()
 {
