@@ -1,6 +1,6 @@
 /* renameop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2021 Ralf Hoffmann.
+ * Copyright (C) 2001-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -18,7 +18,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* $Id: renameop.h,v 1.10 2005/10/19 18:02:45 ralf Exp $ */
 
 #ifndef RENAMEOP_H
 #define RENAMEOP_H
@@ -39,14 +38,23 @@ public:
   const char *getName() override;
   int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
   const char *getDescription() override;
+
+    bool save( Datei * ) override;
+
+    int configure() override;
+  
+    void setAlsoSelectFileExtension( bool nv );
 protected:
   static const char *name;
   // Infos to save
+    bool m_also_select_file_extension = false;
 
   // temp variables
   Lister *startlister;
   
   int normalmoderename( ActionMessage *am );
+
+    int doconfigure( int mode );
 };
 
 #endif
