@@ -1,6 +1,6 @@
 /* searchsettings.cc
  * This file belongs to Worker, a file manager for UNIX/X11.
- * Copyright (C) 2006-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2006-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,6 +24,7 @@
 
 SearchSettings::SearchSettings() : _basedir( "" ),
                                    _matchname( "*" ),
+                                   m_matchname_separator( "," ),
                                    _matchname_re( false ),
                                    _matchname_fullpath( false ),
                                    _matchname_case( false ),
@@ -48,10 +49,12 @@ SearchSettings::SearchSettings() : _basedir( "" ),
     m_max_vfs_depth( 0 ),
     m_match_also_directories( false )
 {
+    create_separated_names();
 }
 
 SearchSettings::SearchSettings( const std::string &basedir,
                                 const std::string &matchname,
+                                const std::string &matchname_separator,
                                 bool matchname_re,
                                 bool matchname_fullpath,
                                 bool matchname_case,
@@ -60,11 +63,14 @@ SearchSettings::SearchSettings( const std::string &basedir,
                                 bool follow_symlinks,
                                 bool search_vfs,
                                 bool search_same_fs,
+                                const std::string &exclude_label,
+                                const std::string &match_expression,
                                 bool check_younger,
                                 const std::string &younger_than,
                                 bool check_older,
                                 const std::string &older_than ) : _basedir( basedir ),
                                                                   _matchname( matchname ),
+                                                                  m_matchname_separator( matchname_separator ),
                                                                   _matchname_re( matchname_re ),
                                                                   _matchname_fullpath( matchname_fullpath ),
                                                                   _matchname_case( matchname_case ),
@@ -74,6 +80,8 @@ SearchSettings::SearchSettings( const std::string &basedir,
                                                                   _follow_symlinks( follow_symlinks ),
                                                                   _search_vfs( search_vfs ),
                                                                   _search_same_fs( search_same_fs ),
+                                                                  m_exclude_label( exclude_label ),
+                                                                  m_match_expression( match_expression ),
                                                                   m_check_younger( check_younger ),
                                                                   m_younger_than( younger_than ),
                                                                   m_check_older( check_older ),
@@ -90,6 +98,7 @@ SearchSettings::SearchSettings( const std::string &basedir,
     m_match_also_directories( false )
 {
     convertDates();
+    create_separated_names();
 }
 
 SearchSettings::~SearchSettings()
@@ -106,6 +115,11 @@ std::string SearchSettings::getMatchName() const
   return _matchname;
 }
 
+std::string SearchSettings::getMatchNameSeparator() const
+{
+    return m_matchname_separator;
+}
+
 bool SearchSettings::getMatchNameRE() const
 {
   return _matchname_re;
@@ -156,6 +170,16 @@ bool SearchSettings::getSearchSameFS() const
   return _search_same_fs;
 }
 
+std::string SearchSettings::getExcludeLabel() const
+{
+    return m_exclude_label;
+}
+
+std::string SearchSettings::getMatchExpression() const
+{
+    return m_match_expression;
+}
+
 bool SearchSettings::getCheckYounger() const
 {
     return m_check_younger;
@@ -312,3 +336,21 @@ bool SearchSettings::getMatchAlsoDirectories() const
 {
     return m_match_also_directories;
 }
+
+const std::vector< std::string > &SearchSettings::getSeparatedMatchNames() const
+{
+    return m_separated_matchnames;
+}
+
+void SearchSettings::create_separated_names()
+{
+    m_separated_matchnames.clear();
+
+    if ( ! m_matchname_separator.empty() ) {
+        AGUIXUtils::split_string( m_separated_matchnames,
+                                  _matchname,
+                                  m_matchname_separator[0] );
+    } else {
+        m_separated_matchnames.push_back( _matchname );
+    }
+}
