@@ -1,6 +1,6 @@
 /* searchthread.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2008,2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2006-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,17 +21,23 @@
 
 #include "searchthread.hh"
 #include "aguix/lowlevelfunc.h"
+#include "aguix/util.h"
 #include "nwc_file.hh"
 #include "pdatei.h"
 #include "worker_locale.h"
 #include "nwc_virtualdir.hh"
 #include <typeinfo>
 #include <set>
+#include "nwcentryselectionstate.hh"
 
 #include <iostream>
 
-SearchThread::SearchThread( std::unique_ptr<NWC::Dir> searchdir ) : _current_mode( SEARCHING ),
-                                                                    _mode_order( SEARCH )
+SearchThread::SearchThread( std::unique_ptr<NWC::Dir> searchdir,
+                            const std::string &skip_initial_dir ) :
+    _current_mode( SEARCHING ),
+    _mode_order( SEARCH ),
+    m_pt( "/" ),
+    m_skip_initial_dir( skip_initial_dir )
 {
   _searchdir = searchdir.release();
 }
@@ -89,6 +95,11 @@ int SearchThread::getNrOfResults()
   return erg;
 }
 
+size_t SearchThread::getExcludedEntries() const
+{
+    return m_excluded_entries;
+}
+
 bool SearchThread::resultsAvailable()
 {
   _result_lock.lock();
@@ -140,8 +151,43 @@ int SearchThread::visit( NWC::File &file, NWC::Dir::WalkControlObj &cobj )
     }
 
     if ( candidate == true ) {
-        if ( checkName( m_sets.getMatchNameFullPath() == true ? file.getFullname() : file.getBasename() ) == true ) {
-            checkContent( file );
+        m_temp_expr_filter_results.clear();
+
+        if ( m_expr_filter ) {
+            NWCEntrySelectionState nwc_entry( file );
+            if ( m_expr_filter->check( nwc_entry ) ) {
+                auto results = m_expr_filter->get_results();
+
+                if ( results.attributes.empty() ) {
+                    m_temp_expr_filter_results.push_back( SearchResult( file.getFullname(), -1 ) );
+                } else {
+                    for ( auto &r : results.attributes ) {
+                        m_temp_expr_filter_results.push_back( SearchResult( file.getFullname(), r ) );
+                    }
+                }
+            } else {
+                candidate = false;
+            }
+        } else {
+            // for an unset expression, assume match with any line so
+            // merge code can just work
+            m_temp_expr_filter_results.push_back( SearchResult( file.getFullname(), -1 ) );
+        }
+
+        if ( candidate && checkName( m_sets.getMatchNameFullPath() == true ? file.getFullname() : file.getBasename() ) == true ) {
+            std::list<SearchResult> temp_results;
+            m_content_checker.checkContent( file, [this]() { return checkInterrupt(); },
+                                            [this]( const std::string &feedback ) {
+                                                _current_action_lock.lock();
+                                                _current_action = feedback;
+                                                _current_action_lock.unlock();
+                                            },
+                                            [&temp_results]( const NWC::File &file,
+                                                             int line_number ) {
+                                                temp_results.push_back( SearchResult( file.getFullname(), line_number ) );
+                                            } );
+
+            mergeResults( m_temp_expr_filter_results, temp_results );
         }
     }
     
@@ -153,6 +199,15 @@ int SearchThread::visit( NWC::File &file, NWC::Dir::WalkControlObj &cobj )
 
 int SearchThread::visitEnterDir( NWC::Dir &dir, NWC::Dir::WalkControlObj &cobj )
 {
+    if ( dir.getFullname() != m_skip_initial_dir ) {
+        const auto node = m_pt.findNode( dir.getFullname() );
+        if ( node && node->getIsExactHit() ) {
+            cobj.setMode( NWC::Dir::WalkControlObj::SKIP );
+            m_excluded_entries++;
+            return 0;
+        }
+    }
+
     bool candidate = true;
     
     _current_action_lock.lock();
@@ -171,7 +226,13 @@ int SearchThread::visitEnterDir( NWC::Dir &dir, NWC::Dir::WalkControlObj &cobj )
         }
 
         if ( candidate == true ) {
-            if ( checkName( m_sets.getMatchNameFullPath() == true ? dir.getFullname() : dir.getBasename() ) == true ) {
+            if ( m_expr_filter ) {
+                NWCEntrySelectionState nwc_entry( dir );
+                if ( ! m_expr_filter->check( nwc_entry ) ) {
+                    candidate = false;
+                }
+            }
+            if ( candidate && checkName( m_sets.getMatchNameFullPath() == true ? dir.getFullname() : dir.getBasename() ) == true ) {
                 if ( m_sets.getMatchContent().length() < 1 ) {
                     _result_lock.lock();
                     _results.push_back( SearchResult( dir.getFullname(), -1, true ) );
@@ -235,158 +296,40 @@ int SearchThread::prepareDirWalk( const NWC::Dir &dir, NWC::Dir::WalkControlObj
 
 bool SearchThread::checkName( const std::string &str )
 {
-  if ( m_sets.getMatchNameRE() == true ) {
-      return re.match( m_sets.getMatchName().c_str(), str.c_str(), ( m_sets.getMatchNameCase() == false ) ? true : false );
-  } else {
-    if ( m_sets.getMatchNameCase() == false ) {
+    bool matched = false;
+
+    for ( const auto &match_name : m_sets.getSeparatedMatchNames() ) {
+        if ( m_sets.getMatchNameRE() == true ) {
+            if ( re.match( match_name.c_str(), str.c_str(), ( m_sets.getMatchNameCase() == false ) ? true : false ) ) {
+                matched = true;
+                break;
+            }
+        } else {
+            if ( m_sets.getMatchNameCase() == false ) {
 #ifdef HAVE_GNU_FNMATCH
-      if ( fnmatch( m_sets.getMatchName().c_str(), str.c_str(), FNM_CASEFOLD ) == 0 ) return true;
+                if ( fnmatch( match_name.c_str(), str.c_str(), FNM_CASEFOLD ) == 0 ) {
+                    matched = true;
+                    break;
+                }
 #else
-      bool matched = false;
-      char *lstr, *pstr;
-      int i;
+                auto lstr = AGUIXUtils::tolower( str );
+                auto pstr = AGUIXUtils::tolower( match_name );
       
-      //TODO this is not UTF8 safe
-      lstr = dupstring( str.c_str() );
-      for ( i = 0; lstr[i] != '\0'; i++ )
-        lstr[i] = tolower( lstr[i] );
-      
-      pstr = dupstring( m_sets.getMatchName().c_str() );
-      for ( i = 0; pstr[i] != '\0'; i++ )
-        pstr[i] = tolower( pstr[i] );
-      
-      if ( fnmatch( pstr, lstr, 0 ) == 0 ) matched = true;
-      _freesafe( lstr );
-      _freesafe( pstr );
-      return matched;
+                if ( fnmatch( pstr.c_str(), lstr.c_str(), 0 ) == 0 ) {
+                    matched = true;
+                    break;
+                }
 #endif
-    } else {
-        if ( fnmatch( m_sets.getMatchName().c_str(), str.c_str(), 0 ) == 0 ) return true;
+            } else {
+                if ( fnmatch( match_name.c_str(), str.c_str(), 0 ) == 0 ) {
+                    matched = true;
+                    break;
+                }
+            }
+        }
     }
-  }
-  return false;                                                                 
-}
 
-int SearchThread::matchContentLine( const NWC::File &file, const char *line, int current_line )
-{
-  int found = 0;
-  if ( re.match( m_sets.getMatchContent().c_str(), line, ( m_sets.getMatchContentCase() == true ) ? false : true ) == true ) {
-    _result_lock.lock();
-    _results.push_back( SearchResult( file.getFullname(), current_line ) );
-    _result_lock.unlock();
-    found = 1;
-  }
-  return found;
-}
-
-struct input_buffer {
-  char buf[4096];
-  int left, pos;
-  PDatei *pd;
-  bool eof_reached;
-};
-
-struct line_buffer {
-  char buf[1024];
-  int size;
-  int used;
-};
-
-static void fillLineBuffer( struct input_buffer &inbuf, struct line_buffer &linebuf, bool &newline_found )
-{
-  char ch;
-  linebuf.used = 0;
-  newline_found = false;
-
-  // fill line buffer from buf (load on demand)
-  while ( linebuf.used < linebuf.size ) {
-
-    // first read from buf until no byte is left or line buffer is full
-    ch = 0;
-    while ( inbuf.left > 0 && linebuf.used < linebuf.size ) {
-      ch = inbuf.buf[inbuf.pos++];
-      inbuf.left--;
-      if ( ch != '\n' ) {
-        if ( ch != 0 ) // ignore null byte
-          linebuf.buf[linebuf.used++] = ch;
-      } else {
-        newline_found = true;
-        break;
-      }
-    }
-    if ( ch == '\n' || linebuf.used == linebuf.size ) break;
-
-    // no newline found and line buffer not filled means no bytes left in buf
-    if ( inbuf.eof_reached == false ) {
-      int len = inbuf.pd->read( inbuf.buf, sizeof(inbuf.buf) );
-      if ( len < 1 ) {
-        // <0 isn't actually eof but it doesn't matter for content matching
-        inbuf.eof_reached = true;
-      } else {
-        inbuf.left = len;
-        inbuf.pos = 0;
-      }
-    } else break;
-  }
-  linebuf.buf[linebuf.used] = '\0';
-}
-
-int SearchThread::checkContent( const NWC::File &file )
-{
-  int matches = 0;
-
-  if ( checkInterrupt() == true )
-    return matches;
-
-  if ( m_sets.getMatchContent().length() < 1 ) {
-    _result_lock.lock();
-    _results.push_back( SearchResult( file.getFullname(), -1 ) );
-    _result_lock.unlock();
-    matches = 1;
-    return matches;
-  }
-
-  if ( file.isReg( true ) == false )
-    return matches;
-
-  struct input_buffer inbuf;
-  struct line_buffer linebuf;
-  PDatei pd;
-
-  inbuf.left = 0;
-  inbuf.pos = 0;
-  inbuf.pd = &pd;
-  inbuf.eof_reached = false;
-
-  linebuf.size = sizeof( linebuf.buf ) - 1;
-  linebuf.used = 0;
-
-  int current_line = 0;
-  int next_line = 0;
-  bool newline_found = false;
-
-  if ( pd.open( file.getFullname().c_str() ) == 0 ) {
-    
-    _current_action_lock.lock();
-    _current_action = catalog.getLocale( 748 );
-    _current_action += file.getFullname();
-    _current_action_lock.unlock();
-    
-    while ( checkInterrupt() == false ) {
-      // stop if no bytes are left in buffer and eof reached
-      if ( inbuf.eof_reached == true && inbuf.left < 1 ) break;
-      
-      current_line = next_line;
-      
-      fillLineBuffer( inbuf, linebuf, newline_found );
-      
-      if ( newline_found == true )
-        next_line = current_line + 1;
-      
-      matches += matchContentLine( file, linebuf.buf, current_line );
-    }
-  }
-  return matches;
+    return matched;
 }
 
 bool SearchThread::checkInterrupt()
@@ -445,4 +388,83 @@ std::string SearchThread::getCurrentActionInfo()
 void SearchThread::setSearchSettings( const SearchSettings &sets )
 {
     m_sets = sets;
+    if ( m_sets.getMatchExpression().empty() ) {
+        m_expr_filter.reset();
+    } else {
+        m_expr_filter = std::make_unique< ExprFilter >( m_sets.getMatchExpression(),
+                                                        ExprFilterOptions{ .disabled_tokens = {
+                                                                "type",
+                                                                "state"
+                                                            },
+                                                                           .follow = sets.getFollowSymlinks()
+                                                        } );
+        m_expr_filter->setCheckInterrupt( [this]() { return checkInterrupt(); } );
+        m_expr_filter->setFeedbackCallback( [this]( const std::string &feedback ) {
+            _current_action_lock.lock();
+            _current_action = feedback;
+            _current_action_lock.unlock();
+        } );
+    }
+    m_content_checker.setRegex( m_sets.getMatchContent(),
+                                m_sets.getMatchContentCase(),
+                                0 );
+}
+
+void SearchThread::setExcludePaths( const std::list< std::string > &paths )
+{
+    m_pt = PathTree< bool >( "/" );
+    for ( const auto &p : paths ) {
+        m_pt.addPath( p, true );
+    }
+}
+
+void SearchThread::mergeResults( const std::list<SearchResult> &l1,
+                                 const std::list<SearchResult> &l2 )
+{
+    if ( l1.empty() || l2.empty() ) return;
+
+    // easy case 1: first list matches all lines, just take second list
+    if ( l1.front().getLineNr() < 0 ) {
+        _result_lock.lock();
+        for ( auto &e : l2 ) {
+            _results.push_back( e );
+        }
+        _result_lock.unlock();
+        return;
+    }
+
+    // easy case 2: second list matches all lines, just take first list
+    if ( l2.front().getLineNr() < 0 ) {
+        _result_lock.lock();
+        for ( auto &e : l1 ) {
+            _results.push_back( e );
+        }
+        _result_lock.unlock();
+        return;
+    }
+
+    // now we actually have to merge. But we can assume the line
+    // numbers are in increasing order
+
+    auto it1 = l1.begin();
+    auto it2 = l2.begin();
+
+    _result_lock.lock();
+
+    for (; it1 != l1.end() && it2 != l2.end();) {
+        if ( it1->getLineNr() == it2->getLineNr() ) {
+            _results.push_back( *it1 );
+            it1++;
+            it2++;
+            continue;
+        }
+
+        if ( it1->getLineNr() < it2->getLineNr() ) {
+            it1++;
+        } else {
+            it2++;
+        }
+    }
+
+    _result_lock.unlock();
 }
