@@ -991,12 +991,12 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
 
 void SetFilterOp::cleanUpPopUp()
 {
-    if ( m_setfilter_cb.getVal() != NULL ) {
+    if ( m_setfilter_cb.get() != NULL ) {
         m_setfilter_cb->resetInstance();
     }
 
     if ( m_worker != NULL &&
-         m_setfilter_cb.getVal() != NULL ) {
+         m_setfilter_cb.get() != NULL ) {
         m_worker->unregisterPopUpCallback( m_setfilter_cb );
     }
 
