@@ -1,6 +1,6 @@
 /* setfilterop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2017 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -42,19 +42,17 @@ public:
   SetFilterOp( const SetFilterOp &other );
   SetFilterOp &operator=( const SetFilterOp &other );
 
-  virtual SetFilterOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-  using FunctionProto::run;
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
+  SetFilterOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
 
   int doconfigure(int mode);
   typedef enum {INCLUDE_FILTER=0,EXCLUDE_FILTER,UNSET_FILTER,UNSET_ALL} setfilterop_mode;
   
-  void setRequestFlags( bool );
   void setFilter( const char * );
   void setFiltermode( setfilterop_mode );
 
@@ -80,7 +78,6 @@ public:
 protected:
   static const char *name;
   // Infos to save
-  bool request_flags;
   std::string filter;
   setfilterop_mode filtermode;
   std::string m_bookmark_label;
