@@ -115,6 +115,9 @@ bool SetSortmodeOp::save(Datei *fh)
     case SORT_EXTENSION:
       fh->configPutPair( "sortby", "extension" );
       break;
+    case SORT_CUSTOM_ATTRIBUTE:
+      fh->configPutPair( "sortby", "customattribute" );
+      break;
     default:
       fh->configPutPair( "sortby", "name" );
       break;
@@ -177,6 +180,7 @@ int SetSortmodeOp::configure()
   cyb[0]->addOption(catalog.getLocale( 553 ));
   cyb[0]->addOption( catalog.getLocale( 907 ) );
   cyb[0]->addOption( catalog.getLocale( 1240 ) );
+  cyb[0]->addOption( catalog.getLocale( 1534 ) );
   cyb[0]->resize(cyb[0]->getMaxSize(),cyb[0]->getHeight());
   if((mode&0xff)==SORT_SIZE) cyb[0]->setOption(1);
   else if((mode&0xff)==SORT_ACCTIME) cyb[0]->setOption(2);
@@ -188,6 +192,7 @@ int SetSortmodeOp::configure()
   else if ( ( mode & 0xff ) == SORT_NLINK ) cyb[0]->setOption( 8 );
   else if ( ( mode & 0xff ) == SORT_PERMISSION ) cyb[0]->setOption( 9 );
   else if ( ( mode & 0xff ) == SORT_EXTENSION ) cyb[0]->setOption( 10 );
+  else if ( ( mode & 0xff ) == SORT_CUSTOM_ATTRIBUTE ) cyb[0]->setOption( 11 );
   else cyb[0]->setOption(0);
   ac1_1->readLimits();
 
@@ -288,6 +293,9 @@ int SetSortmodeOp::configure()
       case 10:
         tsortmode = SORT_EXTENSION;
         break;
+      case 11:
+        tsortmode = SORT_CUSTOM_ATTRIBUTE;
+        break;
       default:
         tsortmode=SORT_NAME;
         break;
@@ -359,6 +367,7 @@ std::string SetSortmodeOp::getStringRepresentation()
     else if ( ( mode & 0xff ) == SORT_NLINK ) res += catalog.getLocale( 553 );
     else if ( ( mode & 0xff ) == SORT_PERMISSION ) res += catalog.getLocale( 907 );
     else if ( ( mode & 0xff ) == SORT_EXTENSION ) res += catalog.getLocale( 1240 );
+    else if ( ( mode & 0xff ) == SORT_CUSTOM_ATTRIBUTE ) res += catalog.getLocale( 1534 );
     else res += catalog.getLocale( 160 );
 
     if ( ( mode & SORT_REVERSE ) == SORT_REVERSE ) {
