@@ -1,6 +1,6 @@
 /* setsortmodeop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -34,15 +34,15 @@ public:
   SetSortmodeOp( const SetSortmodeOp &other );
   SetSortmodeOp &operator=( const SetSortmodeOp &other );
 
-  virtual SetSortmodeOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-  using FunctionProto::run;
-  int run( WPUContext *wpu, ActionMessage* );
-  virtual bool save(Datei*);
-  virtual const char *getDescription();
-  virtual int configure();
-  
+    SetSortmodeOp *duplicate() const override;
+    bool isName(const char *) override;
+    const char *getName() override;
+    int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+    bool save(Datei*) override;
+    const char *getDescription() override;
+    int configure() override;
+    std::string getStringRepresentation() override;
+
   void setMode( int );
   int getMode() const;
   void setSortby( int m );
