@@ -414,12 +414,12 @@ void ShowDirCacheOp::showCacheList( ActionMessage *msg )
                         dir ) != cache_entries.end() ) {
             std::list< RefCount< ArgClass > > args;
 
-            args.push_back( new StringArg( dir ) );
+            args.push_back( RefCount< ArgClass >( new StringArg( dir ) ) );
             lm1->runCommand( "show_cache_entry", args );
         } else {
             std::list< RefCount< ArgClass > > args;
 
-            args.push_back( new StringArg( dir ) );
+            args.push_back( RefCount< ArgClass >( new StringArg( dir ) ) );
             lm1->runCommand( "enter_dir", args );
         }
     }
