@@ -35,7 +35,7 @@ std::string StringCompletion::findCompletion( const std::string matching_string,
                                               const std::list< std::string > &strings )
 {
     if ( strings.empty() ) return "";
-    if ( m_matcher.getVal() == NULL ) return "";
+    if ( m_matcher.get() == NULL ) return "";
 
     std::string base_string = strings.front();
     std::string start_match = matching_string;
@@ -85,7 +85,7 @@ std::string StringCompletion::findCompletion( const std::string matching_string,
 
 size_t StringCompletion::findShortestMatch( const std::string &s )
 {
-    if ( m_matcher.getVal() == NULL ) return 0;
+    if ( m_matcher.get() == NULL ) return 0;
 
     std::string my_s = s;
     int pos = my_s.length();
