@@ -1,6 +1,6 @@
 /* verzeichnis.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2007,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2022 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -298,20 +298,13 @@ const char *Verzeichnis::getDir()
     return _dir_name.c_str();
 }
 
-struct FEsort : public std::binary_function<FileEntry*, FileEntry*, bool>
-{
-    FEsort( int mode ) : m_mode( mode ) {}
-    bool operator() ( FileEntry *fe1, FileEntry *fe2 )
-    {
-        return ( Verzeichnis::sortfunction( fe1, fe2, m_mode ) < 0 ) ? true : false;
-    }
-
-    int m_mode;
-};
-    
 int Verzeichnis::sort( int mode )
 {
-    std::sort( _files.begin(), _files.end(), FEsort( mode ) );
+    std::sort( _files.begin(),
+               _files.end(),
+               [mode]( auto &lhs, auto &rhs ) {
+                   return ( Verzeichnis::sortfunction( lhs, rhs, mode ) < 0 ) ? true : false;
+               });
     return 0;
 }
 
