@@ -148,7 +148,7 @@ void VirtualDirMode::deletef( struct deleteorder *delorder )
                                   if ( actual_row >= 0 ) {
                                       m_lv->setVisMark( actual_row, true );
                                       m_lv->showRow( actual_row, false );
-                                      m_lv->redraw();
+                                      m_lv->timeLimitedRedraw();
                                   }
                               } );
 
