@@ -20,10 +20,10 @@
  */
 
 #include "waitthread.hh"
-#include <aguix/lowlevelfunc.h>
+#include "aguix/lowlevelfunc.h"
 #include "worker.h"
 #include "eventqueue.hh"
-#include <aguix/timeoutstore.hh>
+#include "aguix/timeoutstore.hh"
 
 #include <sys/select.h>
 
