@@ -1,6 +1,6 @@
 /* wcbutton.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006,2011 Ralf Hoffmann.
+ * Copyright (C) 2006-2020 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,7 @@
 
 #include "wcbutton.hh"
 #include "datei.h"
-#include <aguix/util.h>
+#include "aguix/util.h"
 #include "wcdoubleshortkey.hh"
 #include "functionproto.h"
 #include "simplelist.hh"
@@ -31,15 +31,12 @@ WCButton::WCButton()
   text=dupstring("");
   fg=1;
   bg=0;
-  com=new List();
   check=false;
 }
 
 WCButton::~WCButton()
 {
   if(text!=NULL) _freesafe(text);
-  freecoms(com);
-  delete com;
 }
 
 WCButton *WCButton::duplicate() const
@@ -89,28 +86,17 @@ int WCButton::getBG() const
   return bg;
 }
 
-List *WCButton::getComs()
+command_list_t WCButton::getComs()
 {
-  return com;
+    return com;
 }
 
-void WCButton::setComs(List *ncom)
+void WCButton::setComs( const command_list_t &ncoms )
 {
-  freecoms(com);
-  FunctionProto *fp;
-  int id=ncom->initEnum();
-  fp=(FunctionProto*)ncom->getFirstElement(id);
-  while(fp!=NULL) {
-    com->addElement(fp->duplicate());
-    fp=(FunctionProto*)ncom->getNextElement(id);
-  }
-  com->closeEnum(id);  
-}
-
-void WCButton::addCom( const FunctionProto *fp )
-{
-  if ( fp == NULL ) return;
-  com->addElement( fp->duplicate() );
+    com.clear();
+    for ( auto &fp : ncoms ) {
+        com.push_back( std::shared_ptr< FunctionProto >( fp->duplicate() ) );
+    }
 }
 
 void WCButton::setCheck(bool ncheck)
@@ -143,13 +129,9 @@ bool WCButton::save(Datei *fh) const
   fh->configCloseSection(); //shortkeys
 
   fh->configOpenSection( "commands" );
-  id = com->initEnum();
-  FunctionProto *f1 = (FunctionProto*)com->getFirstElement( id );
-  while ( f1 != NULL ) {
-    FunctionProto::presave( fh, f1 );
-    f1 = (FunctionProto*)com->getNextElement( id );
+  for ( auto &f1 : com ) {
+      FunctionProto::presave( fh, f1.get() );
   }
-  com->closeEnum( id );
   fh->configCloseSection(); //commands
   return true;
 }
