@@ -165,7 +165,7 @@ int ClockPanel::saveValues()
   return 0;
 }
 
-void ClockPanel::run( Widget *elem, AGMessage &msg )
+void ClockPanel::run( Widget *elem, const AGMessage &msg )
 {
   if ( msg.type == AG_CYCLEBUTTONCLICKED ) {
     if ( msg.cyclebutton.cyclebutton == cyb ) {
