@@ -1,6 +1,6 @@
 /* wconfig_directory_presets.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2019 Ralf Hoffmann.
+ * Copyright (C) 2019-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -100,6 +100,7 @@ int DirectoryPresetsPanel::create()
     m_sort_cyb->addOption( catalog.getLocale( 553 ) );
     m_sort_cyb->addOption( catalog.getLocale( 907 ) );
     m_sort_cyb->addOption( catalog.getLocale( 1240 ) );
+    m_sort_cyb->addOption( catalog.getLocale( 1534 ) );
     m_sort_cyb->resize( m_sort_cyb->getMaxSize(),
                         m_sort_cyb->getHeight() );
     m_sort_cyb->connect( this );
@@ -363,6 +364,9 @@ void DirectoryPresetsPanel::run( Widget *elem, const AGMessage &msg )
                 case 10:
                     e.sortmode = ( e.sortmode & ~0xff ) | SORT_EXTENSION;
                     break;
+                case 11:
+                    e.sortmode = ( e.sortmode & ~0xff ) | SORT_CUSTOM_ATTRIBUTE;
+                    break;
                 default:
                     e.sortmode = ( e.sortmode & ~0xff ) | SORT_NAME;
                     break;
@@ -409,6 +413,7 @@ void DirectoryPresetsPanel::updateView( const std::string &path )
     else if ( ( e.sortmode & 0xff ) == SORT_NLINK ) m_sort_cyb->setOption( 8 );
     else if ( ( e.sortmode & 0xff ) == SORT_PERMISSION ) m_sort_cyb->setOption( 9 );
     else if ( ( e.sortmode & 0xff ) == SORT_EXTENSION ) m_sort_cyb->setOption( 10 );
+    else if ( ( e.sortmode & 0xff ) == SORT_CUSTOM_ATTRIBUTE ) m_sort_cyb->setOption( 11 );
     else m_sort_cyb->setOption( 0 );
 
     m_reverse_sort_chb->setState( ( ( e.sortmode & SORT_REVERSE ) == SORT_REVERSE ) ? 1 : 0 );
