@@ -130,7 +130,7 @@ int FiletypePanel::saveValues()
   return 0;
 }
 
-void FiletypePanel::run( Widget *elem, AGMessage &msg )
+void FiletypePanel::run( Widget *elem, const AGMessage &msg )
 {
   List *filetypes = tempconfig->getFiletypes();
   int row;
