@@ -1,6 +1,6 @@
 /* wconfig_font.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2017 Ralf Hoffmann.
+ * Copyright (C) 2006-2022 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -40,7 +40,7 @@ int FontPanel::create()
 {
   Panel::create();
 
-  ac1 = setContainer( new AContainer( this, 1, 13 ), true );
+  ac1 = setContainer( new AContainer( this, 1, 15 ), true );
   ac1->setBorderWidth( 5 );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
@@ -91,13 +91,21 @@ int FontPanel::create()
   ts[4]->setFont( _baseconfig.getFont( 4 ).c_str() );
 
   bs[5] = ac1->addWidget( new Button( _aguix, 0, 0, catalog.getLocale( 1208 ), 0 ),
-                          0, 11, AContainer::CO_INCW );
+                          0, 13, AContainer::CO_INCW );
   bs[5]->connect( this );
 
   ts[5] = ac1->addWidget( new Text( _aguix, 0, 0, _baseconfig.getFont( 5 ).c_str() ),
-                          0, 12, AContainer::CO_INCW );
+                          0, 14, AContainer::CO_INCW );
   ts[5]->setFont( _baseconfig.getFont( 5 ).c_str() );
 
+  bs[6] = ac1->addWidget( new Button( _aguix, 0, 0, catalog.getLocale( 1453 ), 0 ),
+                          0, 11, AContainer::CO_INCW );
+  bs[6]->connect( this );
+
+  ts[6] = ac1->addWidget( new Text( _aguix, 0, 0, _baseconfig.getFont( 6 ).c_str() ),
+                          0, 12, AContainer::CO_INCW );
+  ts[6]->setFont( _baseconfig.getFont( 6 ).c_str() );
+
   ac1->readLimits();
   contMaximize( true );
   return 0;
@@ -111,6 +119,7 @@ int FontPanel::saveValues()
   _baseconfig.setFont( 3, ts[3]->getText() );
   _baseconfig.setFont( 4, ts[4]->getText() );
   _baseconfig.setFont( 5, ts[5]->getText() );
+  _baseconfig.setFont( 6, ts[6]->getText() );
   return 0;
 }
 
@@ -120,7 +129,7 @@ void FontPanel::run( Widget *elem, const AGMessage &msg )
   char *tstr;
 
   if ( msg.type == AG_BUTTONCLICKED ) {
-    for ( i = 0; i < 6; i++ ) {
+    for ( i = 0; i < 7; i++ ) {
       if ( msg.button.button == bs[i] ) {
         tstr = NULL;
         erg = freq->request( ts[i]->getText(), &tstr );
