@@ -203,7 +203,7 @@ int ListViewPanel::saveValues()
   return 0;
 }
 
-void ListViewPanel::run( Widget *elem, AGMessage &msg )
+void ListViewPanel::run( Widget *elem, const AGMessage &msg )
 {
   int side;
   int trow;
