@@ -1,6 +1,6 @@
 /* wconfig_path.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2014 Ralf Hoffmann.
+ * Copyright (C) 2006-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -38,7 +38,7 @@ PathPanel::PathPanel( AWindow &basewin, WConfig &baseconfig ) : WConfigPanel( ba
   maxbank = _baseconfig.getPaths()->size()/rows;
 
   extmode = false;
-  mode = 0;
+  m_mode = 0;
   selindex = -1;
 }
 
@@ -232,39 +232,39 @@ void PathPanel::run( Widget *elem, const AGMessage &msg )
 	showPathBank();
       }
     } else if ( msg.button.button == copyb ) {
-      if ( ( mode == 1 ) || ( mode == 2 ) ) {
+      if ( ( m_mode == 1 ) || ( m_mode == 2 ) ) {
 	sbsb->setText( catalog.getLocale( 76 ) );
-	mode = 0;
+	setMode( 0 );
       } else {
 	sbsb->setText( catalog.getLocale( 77 ) );
-	mode = 1;
+	setMode( 1 );
       }
     } else if ( msg.button.button == swapb ) {
-      if ( ( mode == 3 ) || ( mode == 4 ) ) {
+      if ( ( m_mode == 3 ) || ( m_mode == 4 ) ) {
 	sbsb->setText( catalog.getLocale( 76 ) );
-	mode = 0;
+	setMode( 0 );
       } else {
 	sbsb->setText( catalog.getLocale( 79 ) );
-	mode = 3;
+	setMode( 3 );
       }
     } else if ( msg.button.button == delb ) {
-      if ( mode == 5 ) {
+      if ( m_mode == 5 ) {
 	sbsb->setText( catalog.getLocale( 76 ) );
-	mode = 0;
+	setMode( 0 );
       } else {
 	sbsb->setText( catalog.getLocale( 81 ) );
-	mode = 5;
+	setMode( 5 );
       }
     } else {
       for ( i2 = 0; i2 < rows; i2++ ) {
 	if ( msg.button.button == pbs[i2] ) {
 	  int s1 = i2 + banknr * rows;
 	  WCPath *p1, *p2;
-	  switch ( mode ) {
+	  switch ( m_mode ) {
 	  case 1:
 	    selindex = s1;
 	    sbsb->setText( catalog.getLocale( 78 ) );
-	    mode = 2;
+	    setMode( 2 );
 	    break;
 	  case 2:
 	    // copy selindex -> s1
@@ -278,16 +278,16 @@ void PathPanel::run( Widget *elem, const AGMessage &msg )
 	    showPathBank();
 	    if ( extmode == true ) {
 	      sbsb->setText( catalog.getLocale( 77 ) );
-	      mode = 1;
+	      setMode( 1 );
 	    } else {
 	      sbsb->setText( catalog.getLocale( 76 ) );
-	      mode = 0;
+	      setMode( 0 );
 	    }
 	    break;
 	  case 3:
 	    selindex = s1;
 	    sbsb->setText( catalog.getLocale( 80 ) );
-	    mode = 4;
+	    setMode( 4 );
 	    break;
 	  case 4:
 	    // swap s1 <-> selindex
@@ -297,10 +297,10 @@ void PathPanel::run( Widget *elem, const AGMessage &msg )
 	    showPathBank();
 	    if ( extmode == true ) {
 	      sbsb->setText( catalog.getLocale( 79 ) );
-	      mode = 3;
+	      setMode( 3 );
 	    } else {
 	      sbsb->setText( catalog.getLocale( 76 ) );
-	      mode = 0;
+	      setMode( 0 );
 	    }
 	    break;
 	  case 5:
@@ -311,7 +311,7 @@ void PathPanel::run( Widget *elem, const AGMessage &msg )
 	    showPathBank();
 	    if ( extmode == false ) {
 	      sbsb->setText( catalog.getLocale( 76 ) );
-	      mode = 0;
+	      setMode( 0 );
 	    }
 	    break;
 	  default:
@@ -335,7 +335,7 @@ void PathPanel::run( Widget *elem, const AGMessage &msg )
   } else if ( msg.type == AG_KEYPRESSED ) {
     if ( msg.key.key == XK_Escape ) {
       sbsb->setText( catalog.getLocale( 76 ) );
-      mode = 0;
+      setMode( 0 );
     }
   }
 }
@@ -393,3 +393,19 @@ void PathPanel::updateBankStr()
   sprintf( bankstr, "%2d", maxbank );
   maxbankt->setText( bankstr );
 }
+
+bool PathPanel::escapeInUse()
+{
+    // return current value and check if mode is still in use
+    bool res = m_escape_active;
+
+    if ( m_mode == 0 ) m_escape_active = false;
+
+    return res;
+}
+
+void PathPanel::setMode( int m )
+{
+    if ( m != 0 ) m_escape_active = true;
+    m_mode = m;
+}
