@@ -23,8 +23,8 @@
 #include "wconfig.h"
 #include "worker.h"
 #include "worker_locale.h"
-#include <aguix/stringgadget.h>
-#include <aguix/cyclebutton.h>
+#include "aguix/stringgadget.h"
+#include "aguix/cyclebutton.h"
 
 StartPanel::StartPanel( AWindow &basewin, WConfig &baseconfig ) : WConfigPanel( basewin, baseconfig )
 {
