@@ -1,6 +1,6 @@
 /* worker_commands.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2021 Ralf Hoffmann.
+ * Copyright (C) 2001-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -25,7 +25,7 @@
 
 int Worker::getNrOfCommands()
 {
-  return 59;
+  return 60;
 }
 
 std::shared_ptr< FunctionProto > Worker::getCommand4ID(int id)
@@ -202,6 +202,9 @@ std::shared_ptr< FunctionProto > Worker::getCommand4ID(int id)
     case 58:
       fpr = new ViewCommandLogOp();
       break;
+    case 59:
+      fpr = new ActivateTextViewModeOp();
+      break;
     default:
       fpr=new DNDAction();
   }
@@ -270,6 +273,7 @@ int Worker::getID4Command(FunctionProto *fpr)
   else if ( fpr->isName( ModifyTabsOp::name ) == true ) type = 56;
   else if ( fpr->isName( HelpOp::name ) == true ) type = 57;
   else if ( fpr->isName( ViewCommandLogOp::name ) == true ) type = 58;
+  else if ( fpr->isName( ActivateTextViewModeOp::name ) == true ) type = 59;
   return type;
 }
 
