@@ -1,6 +1,6 @@
 /* worker_types.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2003-2019 Ralf Hoffmann.
+ * Copyright (C) 2003-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -35,7 +35,8 @@ struct WorkerTypes::availListCols_t WorkerTypes::availListCols[] = {
   { WorkerTypes::LISTCOL_NLINK, 549 },
   { WorkerTypes::LISTCOL_BLOCKS, 550 },
   { WorkerTypes::LISTCOL_SIZEH, 914 },
-  { WorkerTypes::LISTCOL_EXTENSION, 1239 }
+  { WorkerTypes::LISTCOL_EXTENSION, 1239 },
+  { WorkerTypes::LISTCOL_CUSTOM_ATTR, 1534 }
 };
 
 unsigned int WorkerTypes::getAvailListColsSize()
