@@ -1,6 +1,6 @@
 /* wpucontext.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2021 Ralf Hoffmann.
+ * Copyright (C) 2002-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -39,14 +39,7 @@
 #include "listermode.h"
 #include "nwc_path.hh"
 #include "virtualdirmode.hh"
-
-WPUStack::WPUStack()
-{
-}
-
-WPUStack::~WPUStack()
-{
-}
+#include <cinttypes>
 
 void WPUStack::push( const std::string &elem )
 {
@@ -108,8 +101,6 @@ WPUContext::WPUContext( Worker *w, const command_list_t &coms, ActionMessage *ms
 {
   struct wpucontext_deftokens *p;
 
-  nr_of_stacks = 0;
-  stacks = NULL;
   pc = 0;
 
   acoms = coms;
@@ -179,19 +170,36 @@ WPUContext::WPUContext( Worker *w, const command_list_t &coms, ActionMessage *ms
             extlist[0]->createExtList( activemode, msg, false );
             extlist[0]->setTakeDirs( m_take_dirs );
         }
+      } else if ( msg->mode == ActionMessage::AM_MODE_ON_DEMAND ) {
+          if ( activemode != NULL ) {
+              extlist[0] = new NMExtList();
+              extlist[0]->createExtList( activemode, msg, false );
+              extlist[0]->setTakeDirs( m_take_dirs );
+          }
+          if ( nonactivemode != NULL ) {
+              extlist[1] = new NMExtList();
+              extlist[1]->createExtList( nonactivemode, msg, false );
+              extlist[1]->setTakeDirs( m_take_dirs );
+          }
       }
     }
   }
 }
 
+WPUContext::WPUContext( WPUContext &other,
+                        ActionMessage *msg ) :
+    WPUContext( msg->getWorker(),
+                {},
+                msg )
+{
+    m_stacks = other.getStacks();
+}
+
+
 WPUContext::~WPUContext()
 {
   closeWin();
 
-  for ( int i = 0; i < nr_of_stacks; i++ ) {
-    delete stacks[i];
-  }
-  delete [] stacks;
   if ( extlist[0] != NULL ) {
     delete extlist[0];
   }
@@ -228,50 +236,50 @@ int WPUContext::continueAtLabel( const char *label )
 
 void WPUContext::push( int nr, std::string elem )
 {
-  if ( nr >= nr_of_stacks ) buildStack( nr );
-  stacks[ nr ]->push( elem );
+    buildStack( nr );
+    (*m_stacks)[ nr ].push( elem );
 }
 
 std::string WPUContext::pop( int nr )
 {
-  if ( nr >= nr_of_stacks ) buildStack( nr );
-  if ( stacks[ nr ]->size() < 1 ) {
-    lasterror = WPU_LASTERROR_STACKEMPTY;
-    // we can still call top because it will return ""
-  }
-  return stacks[ nr ]->pop();
+    buildStack( nr );
+    if ( (*m_stacks)[ nr ].size() < 1 ) {
+        lasterror = WPU_LASTERROR_STACKEMPTY;
+        // we can still call top because it will return ""
+    }
+    return (*m_stacks)[ nr ].pop();
 }
 
 std::string WPUContext::top( int nr )
 {
-  if ( nr >= nr_of_stacks ) buildStack( nr );
-  if ( stacks[ nr ]->size() < 1 ) {
-    lasterror = WPU_LASTERROR_STACKEMPTY;
-    // we can still call top because it will return ""
-  }
-  return stacks[ nr ]->top();
+    buildStack( nr );
+    if ( (*m_stacks)[ nr ].size() < 1 ) {
+        lasterror = WPU_LASTERROR_STACKEMPTY;
+        // we can still call top because it will return ""
+    }
+    return (*m_stacks)[ nr ].top();
 }
 
 std::string WPUContext::stackEntry( int nr, int pos )
 {
-    if ( nr >= nr_of_stacks ) {
+    if ( ! m_stacks || nr >= (int)m_stacks->size() ) {
         lasterror = WPU_LASTERROR_INVALID_STACK;
         return "";
     }
 
-    return stacks[ nr ]->stackEntry( pos );
+    return (*m_stacks)[ nr ].stackEntry( pos );
 }
 
 bool WPUContext::isEmpty( int nr )
 {
-  if ( nr >= nr_of_stacks ) buildStack( nr );
-  return stacks[ nr ]->isEmpty();
+    buildStack( nr );
+    return (*m_stacks)[ nr ].isEmpty();
 }
 
 int WPUContext::size( int nr )
 {
-  if ( nr >= nr_of_stacks ) buildStack( nr );
-  return (int)stacks[ nr ]->size();
+    buildStack( nr );
+    return (int)(*m_stacks)[ nr ].size();
 }
 
 int WPUContext::next( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
@@ -310,19 +318,13 @@ int WPUContext::next( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
  */
 void WPUContext::buildStack( int nr )
 {
-  WPUStack **newstacks;
-  int i;
-  
-  if ( nr >= nr_of_stacks ) {
-    newstacks = new WPUStack*[ nr + 1 ];
-    for ( i = 0; i < nr_of_stacks; i++ )
-      newstacks[ i ] = stacks[ i ];
-    for ( i = nr_of_stacks; i < ( nr + 1); i++ )
-      newstacks[ i ] = new WPUStack();
-    delete [] stacks;
-    stacks = newstacks;
-    nr_of_stacks = nr + 1;
-  }
+    if ( ! m_stacks ) {
+        m_stacks = std::make_shared< std::vector< WPUStack > >();
+    }
+
+    if ( nr < (int)m_stacks->size() ) return;
+
+    m_stacks->resize( nr + 1 );
 }
 
 int WPUContext::openWin()
@@ -385,35 +387,28 @@ WPUContext::parse_com_str_t WPUContext::parse( const char *str1, std::string &re
     parse_com_str_t res = PARSE_ERROR;
   
     if ( worker != NULL ) {
-        buildExtList();
-
-        int old_size = 0;
-        std::string fullname;
-
-        if ( extlist[0] != NULL &&
-             extlist[0]->getSize() > 0 ) {
-            int id = extlist[0]->initEnum();
-            NM_extern_fe *efe1 = (NM_extern_fe*)extlist[0]->getFirstElement( id );
-
-            old_size = extlist[0]->getSize();
-            fullname = efe1->getFullname( false );
-
-            extlist[0]->closeEnum( id );
-        }
-
         res = parseComStr( str1, maxlen,
                            quote, return_str);
 
-        if ( persist != PERSIST_NOTHING ) {
-            if ( extlist[0] == NULL ||
-                 extlist[0]->getSize() != old_size ) {
-                worker->storePathPersIfInProgress( fullname );
+        if ( extlist[0] != NULL ) {
+            if ( extlist[0]->modified() ) {
+                if ( persist != PERSIST_NOTHING ) {
+                    worker->storePathPersIfInProgress( extlist[0]->getFirstFullname() );
+                }
 
-                m_pathdb_filename = fullname;
-            } else if ( persist == PERSIST_ALL ) {
-                worker->storePathPersIfInProgress( NWC::Path::dirname( fullname ) );
+                m_pathdb_filename = extlist[0]->getFirstFullname();
+            } else {
+                if ( extlist[0]->getFirstFullname().empty() ) {
+                    int id = extlist[0]->initEnum();
+                    extlist[0]->getFirstElement( id );
+                    extlist[0]->closeEnum( id );
+                }
 
-                m_pathdb_filename = fullname;
+                if ( persist == PERSIST_ALL ) {
+                    worker->storePathPersIfInProgress( NWC::Path::dirname( extlist[0]->getFirstFullname() ) );
+                }
+
+                m_pathdb_filename = extlist[0]->getFirstFullname();
             }
         }
     }
@@ -920,7 +915,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
                 _freesafe( reqstr );
 
                 RefCount<AFontWidth> lencalc( new AFontWidth( worker->getAGUIX(), NULL ) );
-                TextStorageString ts( str1, lencalc );
+                auto ts = std::make_shared< TextStorageString >( str1, lencalc );
 
                 worker->getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
               }
@@ -989,7 +984,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
                 _freesafe( reqstr );
 
                 RefCount<AFontWidth> lencalc( new AFontWidth( worker->getAGUIX(), NULL ) );
-                TextStorageString ts( str1, lencalc );
+                auto ts = std::make_shared< TextStorageString >( str1, lencalc );
 
                 worker->getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
               }
@@ -1103,8 +1098,7 @@ int WPUContext::parse_if( const char *str)
 bool WPUContext::filelistEmpty( bool other )
 {
   buildExtList();
-  if ( extlist[ ( other == true ) ? 1 : 0 ]->getRealSize() < 1 ) return true;
-  return false;
+  return extlist[ ( other == true ) ? 1 : 0 ]->empty();
 }
 
 void WPUContext::buildExtList()
@@ -1252,16 +1246,17 @@ const char *WPUContext::getBaseDir()
     return m_base_dir.c_str();
 }
 
-int WPUContext::filelistSize( bool other )
+bool WPUContext::filelistModified( bool other, bool reset )
 {
-  buildExtList();
-  return extlist[ ( other == true ) ? 1 : 0 ]->getSize();
-}
+    buildExtList();
 
-int WPUContext::filelistRealSize( bool other )
-{
-  buildExtList();
-  return extlist[ ( other == true ) ? 1 : 0 ]->getRealSize();
+    bool m = extlist[ ( other == true ) ? 1 : 0 ]->modified();
+
+    if ( reset ) {
+        extlist[ ( other == true ) ? 1 : 0 ]->resetModifedFlag();
+    }
+
+    return m;
 }
 
 void WPUContext::setTakeDirs( bool nv )
@@ -1381,6 +1376,8 @@ void WPUContext::calculateFilesizes()
 
     loff_t sum = 0;
 
+    buildExtList();
+
     for ( int i = 0; i < 2; i++ ) {
         if ( extlist[i] ) {
             int id = extlist[i]->initEnum();
@@ -1670,6 +1667,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                         Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                 cwd.c_str(),
+                                                "",
                                                 cwd.c_str() );
                     }
                 }
@@ -1685,6 +1683,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                         Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                 cwd.c_str(),
+                                                "",
                                                 cwd.c_str() );
                     }
                 }
@@ -1705,6 +1704,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                         Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                 cwd.c_str(),
+                                                "",
                                                 cwd.c_str() );
                     }
                 }
@@ -1725,6 +1725,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                         Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                 cwd.c_str(),
+                                                "",
                                                 cwd.c_str() );
                     }
                 }
@@ -1740,6 +1741,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                         Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                 cwd.c_str(),
+                                                "",
                                                 cwd.c_str() );
                     }
                 }
@@ -1751,62 +1753,126 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                         ende = true;
                     }
                 }
+            } else if ( strncmp( buf1, "random", 6 ) == 0 ) {
+                long r = lrand48();
+                std::string s = AGUIXUtils::formatStringToString( "%" PRIu32, (uint32_t)r );
+                if ( ! res_cli_str.add_str( s, force_noquoting ) ) {
+                    ende = true;
+                }
+            } else if ( strncmp( buf1, "currenttime", 11 ) == 0 ) {
+                time_t now = time( NULL );
+                std::string s = AGUIXUtils::formatStringToString( "%" PRIu64, (uint64_t)now );
+                if ( ! res_cli_str.add_str( s, force_noquoting ) ) {
+                    ende = true;
+                }
             } else {
                 // file flag or unknown
                 bool otherside, nounselect, noext;
-                int filemode, tpos, tmode;
+                int tpos;
 
-                filemode = -1;
+                enum parse_mode {
+                    FLAG_PARSE_START = 0,
+                    FLAG_PARSE_SIDE_DONE = 1,
+                    FLAG_PARSE_SELECT_DONE = 2,
+                    FLAG_PARSE_FILE_DONE = 3,
+                    FLAG_PARSE_TIME = 4,
+                    FLAG_PARSE_DONE = 5,
+                    FLAG_PARSE_INVALID = 6
+                } tmode;
+
+                enum file_mode {
+                    FILE_MODE_UNSET = -1,
+                    FILE_MODE_FIRST_BASENAME = 0,
+                    FILE_MODE_ALL_BASENAME = 1,
+                    FILE_MODE_FIRST_FULLNAME = 2,
+                    FILE_MODE_ALL_FULLNAME = 3,
+                    FILE_MODE_TEMPNAME = 4,
+                    FILE_MODE_EXTENSION = 5,
+                    FILE_MODE_SIZE = 6,
+                    FILE_MODE_MTIME = 7,
+                    FILE_MODE_CTIME = 8
+                } filemode;
+
+                filemode = FILE_MODE_UNSET;
                 otherside = false;
                 nounselect = false;
                 noext = false;
                 tpos = 0;
-                tmode = 0;
-                for( ; tmode < 4 && tpos < 4; ) {
+                tmode = FLAG_PARSE_START;
+                for( ; tmode < FLAG_PARSE_DONE && tpos < 4; ) {
                     ch = buf1[ tpos++ ];
                     if ( ch == 'o' ) {
-                        if ( tmode < 1 ) {
+                        if ( tmode == FLAG_PARSE_START ) {
                             otherside = true;
-                            tmode = 1;
+                            tmode = FLAG_PARSE_SIDE_DONE;
                         } else {
                             // an "o" at the wrong position
-                            tmode = 6;
+                            tmode = FLAG_PARSE_INVALID;
                         }
                     } else if ( ch == 'u' ) {
-                        if ( tmode < 2 ) {
+                        if ( tmode < FLAG_PARSE_SELECT_DONE ) {
                             nounselect = true;
-                            tmode = 2;
+                            tmode = FLAG_PARSE_SELECT_DONE;
                         } else {
-                            tmode = 6;
+                            tmode = FLAG_PARSE_INVALID;
                         }
-                    } else if ( ch == 'f' && tmode < 3 ) {
-                        filemode = 0;
-                        tmode = 3;
-                    } else if ( ch == 'a' && tmode < 3 ) {
-                        filemode = 1;
-                        tmode = 3;
-                    } else if ( ch == 'F' && tmode < 3 ) {
-                        filemode = 2;
-                        tmode = 3;
-                    } else if ( ch == 'A' && tmode < 3 ) {
-                        filemode = 3;
-                        tmode = 3;
-                    } else if ( ch == 'E' && tmode == 3 ) {
+                    } else if ( ch == 'f' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        filemode = FILE_MODE_FIRST_BASENAME;
+                        tmode = FLAG_PARSE_FILE_DONE;
+                    } else if ( ch == 'a' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        filemode = FILE_MODE_ALL_BASENAME;
+                        tmode = FLAG_PARSE_FILE_DONE;
+                    } else if ( ch == 'F' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        filemode = FILE_MODE_FIRST_FULLNAME;
+                        tmode = FLAG_PARSE_FILE_DONE;
+                    } else if ( ch == 'A' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        filemode = FILE_MODE_ALL_FULLNAME;
+                        tmode = FLAG_PARSE_FILE_DONE;
+                    } else if ( ch == 'E' && tmode == FLAG_PARSE_FILE_DONE ) {
                         noext = true;
-                        tmode = 4;
-                    } else if ( ch == 't' && tmode < 3 ) {
-                        filemode = 4;
-                        tmode = 4;  // no E allowed so directly change to mode 4
-                    } else if ( ch == '\0' ) tmode = 4;
+                        tmode = FLAG_PARSE_DONE;
+                    } else if ( ch == 'E' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        // force no unselect for now
+                        nounselect = true;
+                        filemode = FILE_MODE_EXTENSION;
+                        tmode = FLAG_PARSE_DONE;
+                    } else if ( ch == 's' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        // force no unselect for now
+                        nounselect = true;
+                        filemode = FILE_MODE_SIZE;
+                        tmode = FLAG_PARSE_DONE;
+                    } else if ( ch == 'T' && tmode <= FLAG_PARSE_FILE_DONE ) {
+                        tmode = FLAG_PARSE_TIME;
+                    } else if ( ch == 'm' && tmode == FLAG_PARSE_TIME ) {
+                        // force no unselect for now
+                        nounselect = true;
+                        filemode = FILE_MODE_MTIME;
+                        tmode = FLAG_PARSE_DONE;
+                    } else if ( ch == 'c' && tmode == FLAG_PARSE_TIME ) {
+                        // force no unselect for now
+                        nounselect = true;
+                        filemode = FILE_MODE_CTIME;
+                        tmode = FLAG_PARSE_DONE;
+                    } else if ( ch == 't' && tmode < FLAG_PARSE_FILE_DONE ) {
+                        filemode = FILE_MODE_TEMPNAME;
+                        tmode = FLAG_PARSE_DONE;  // no E allowed so directly change to mode 4
+                    } else if ( ch == '\0' ) {
+                        tmode = FLAG_PARSE_DONE;
+                    } else {
+                        filemode = FILE_MODE_UNSET;
+                        tmode = FLAG_PARSE_INVALID;
+                    }
                 }
-                if (  filemode >= 0 && filemode < 5 ) {
+                if ( filemode >= FILE_MODE_FIRST_BASENAME && filemode <= FILE_MODE_CTIME ) {
                     // we have a valid flag
                     bool takefirst = true;
                     NM_extern_fe *efe1;
                     NMExtList *l;
                     bool efeskip;
 
-                    if ( filemode == 1 || filemode == 3 ) takefirst = false;
+                    buildExtList();
+                    
+                    if ( filemode == FILE_MODE_ALL_BASENAME || filemode == FILE_MODE_ALL_FULLNAME ) takefirst = false;
                     if ( otherside == true ) l = extlist[1];
                     else l = extlist[0];
                     id = l->initEnum();
@@ -1832,12 +1898,70 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                                     name_cstring = name_string.c_str();
                                 }
                             } else {
-                                if ( filemode == 4 ) {
+                                if ( filemode == FILE_MODE_TEMPNAME ) {
                                     name_string = getTempName4File( efe1->getFullname( false ) );
                                     if ( ! name_string.empty() ) {
                                         name_cstring = name_string.c_str();
                                     }
-                                } else if ( filemode < 2 ) {
+                                } else if ( filemode == FILE_MODE_EXTENSION ) {
+                                    auto extension = efe1->getExtension();
+
+                                    if ( extension ) {
+                                        name_string = *extension;
+                                    } else {
+                                        name_string.clear();
+                                    }
+                                    name_cstring = name_string.c_str();
+                                } else if ( filemode == FILE_MODE_SIZE ) {
+                                    loff_t s = 0;
+                                    const FileEntry *fe = efe1->getFE();
+
+                                    if ( fe ) {
+                                        if ( fe->isLink && ! fe->isCorrupt ) {
+                                            s = fe->dsize();
+                                        } else {
+                                            s = fe->size();
+                                        }
+                                    }
+
+                                    name_string = AGUIXUtils::formatStringToString( "%" PRIu64, (uint64_t)s );
+                                    if ( ! name_string.empty() ) {
+                                        name_cstring = name_string.c_str();
+                                    }
+                                } else if ( filemode == FILE_MODE_MTIME ) {
+                                    time_t t = 0;
+                                    const FileEntry *fe = efe1->getFE();
+
+                                    if ( fe ) {
+                                        if ( fe->isLink && ! fe->isCorrupt ) {
+                                            t = fe->dlastmod();
+                                        } else {
+                                            t = fe->lastmod();
+                                        }
+                                    }
+
+                                    name_string = AGUIXUtils::formatStringToString( "%" PRIu64, (uint64_t)t );
+                                    if ( ! name_string.empty() ) {
+                                        name_cstring = name_string.c_str();
+                                    }
+                                } else if ( filemode == FILE_MODE_CTIME ) {
+                                    time_t t = 0;
+                                    const FileEntry *fe = efe1->getFE();
+
+                                    if ( fe ) {
+                                        if ( fe->isLink && ! fe->isCorrupt ) {
+                                            t = fe->dlastchange();
+                                        } else {
+                                            t = fe->lastchange();
+                                        }
+                                    }
+
+                                    name_string = AGUIXUtils::formatStringToString( "%" PRIu64, (uint64_t)t );
+                                    if ( ! name_string.empty() ) {
+                                        name_cstring = name_string.c_str();
+                                    }
+                                } else if ( filemode == FILE_MODE_FIRST_BASENAME ||
+                                            filemode == FILE_MODE_ALL_BASENAME ) {
                                     std::string cwd;
 
                                     if ( use_extended_basenames ) {
@@ -1880,6 +2004,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
 
                                 Worker::pushCommandLog( catalog.getLocale( 1416 ),
                                                         efe1->getFullname( false ),
+                                                        "",
                                                         efe1->getFullname( false ) );
                             }               
                             if ( ende == true ) break;
@@ -1900,9 +2025,17 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
                                     }
                                 }
                                 delete efe1;
-                                efe1 = (NM_extern_fe*)l->getFirstElement( id );
+                                if ( ! takefirst ) {
+                                    efe1 = (NM_extern_fe*)l->getFirstElement( id );
+                                } else {
+                                    efe1 = NULL;
+                                }
                             } else {
-                                efe1 = (NM_extern_fe*)l->getNextElement( id );
+                                if ( ! takefirst ) {
+                                    efe1 = (NM_extern_fe*)l->getNextElement( id );
+                                } else {
+                                    efe1 = NULL;
+                                }
                             }
                             if ( efe1 == NULL ) break;
                             if ( ( efe1->getDirFinished() == 0 ) ||
@@ -1933,7 +2066,7 @@ WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
         std::string message = catalog.getLocale( 938 );
         message += sstr;
 
-        TextStorageString ts( message, lencalc );
+        auto ts = std::make_shared< TextStorageString >( message, lencalc );
 
         std::string buttons = catalog.getLocale( 629 );
         buttons += "|";
@@ -2064,3 +2197,8 @@ const std::string &WPUContext::getPathDBFilename() const
 {
     return m_pathdb_filename;
 }
+
+std::shared_ptr< std::vector< WPUStack > > WPUContext::getStacks()
+{
+    return m_stacks;
+}
